'''
Sempy: A Python implementation of the spectral element method.
Copyright (C) 2011,  Stian Jensen.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
''' 
import sempy
from prk import PRK
import numpy as np
import scipy as sp
import scipy.sparse.linalg as spl
import pickle



def euler():
     """
     Euler method 1st order.
     """
     alpha = np.zeros(2) #: time derivative
     beta  = np.zeros(2) #: implicit terms
     gamma = np.zeros(2) #: explicit terms
     alpha[1],alpha[0] = 1.0,-1.0 #: time derivative
     beta[1], beta[0] = 1.0, 0.0  #: implicit terms
     gamma[1],gamma[0] = 0.0, 1.0  #: explicit terms
     levels = 2 
     return alpha,beta,gamma,levels

def cnab2():
     """ Crank-Nicolson/Adams-Bashforth 2nd order. 
         return: alpha,beta, gamma,levels 
     """
     alpha,beta,gamma =np.zeros(3),np.zeros(3),np.zeros(3)
     alpha[2],alpha[1],alpha[0]=1.0,-1.0,0.0
     beta[2] ,beta[1], beta [0]=0.5,0.5,0.0
     gamma[2],gamma[1],gamma[0]=0.0,1.5,-0.5
     levels=3
     return alpha,beta,gamma,levels

def cn():
     """ Crank-Nicolson/Adams-Bashforth 2nd order. 
         return: alpha,beta, gamma,levels 
     """
     alpha,beta,gamma =np.zeros(2),np.zeros(2),np.zeros(2)
     alpha[1],alpha[0]=1.0,-1.0
     beta[1], beta [0]=0.5,0.5
     #gamma[2],gamma[1],gamma[0]=0.0,1.5,-0.5
     levels=2
     return alpha,beta,gamma,levels
 
def mcnab2():
     """Modified Crank-Nicolson/Adams-Bashforth 2nd order."""
     alpha,beta,gamma =np.zeros(3),np.zeros(3),np.zeros(3)
     alpha[2],alpha[1],alpha[0]=1.0,-1.0,0.0
     beta[2] ,beta[1], beta [0]=9.0/16.0,6.0/16.0,1.0/16
     gamma[2],gamma[1],gamma[0]=0.0,1.5,-0.5
     levels=3
     return alpha,beta,gamma,levels

def cnlf():
     """Crank-Nicolson/Leapfrog 2nd order"""
     alpha,beta,gamma =np.zeros(3),np.zeros(3),np.zeros(3)
     alpha[2],alpha[1],alpha[0]=1.0,0.0,-1.0
     beta[2] ,beta[1], beta [0]=1.0,0.0,1.0
     gamma[2],gamma[1],gamma[0]=0.0,2.0,0.0
     levels=3
     return alpha,beta,gamma,levels

def bdfex2():
     """Backward differentiation formula/extrapolation 2nd order"""
     alpha,beta,gamma =np.zeros(3),np.zeros(3),np.zeros(3)
     alpha[2],alpha[1],alpha[0]=1.5,-2.0,0.5
     beta[2] ,beta[1], beta [0]=1.0,0.0,0.0
     gamma[2],gamma[1],gamma[0]=0.0,2.0,-1.0
     levels=3
     return alpha,beta,gamma,levels

def bdfex3():
     """Backward differentiation formula/extrapolation 2nd order"""
     alpha, beta, gamma = np.zeros(4), np.zeros(4), np.zeros(4)
     alpha[3] =  11.0/6.0
     alpha[2] = -3.0
     alpha[1] =  3.0/2.0
     alpha[0] = -1.0/3.0
     beta[3], beta[2], beta[1], beta[0] =  1.0, 0.0, 0.0, 0.0
     gamma[3], gamma[2], gamma[1], gamma[0] = 0.0, 3.0, -3.0, 1.0
     levels = 4
     return alpha, beta, gamma, levels

def __emptyfunc__(y,t):
   """This is an empty func."""
   v = np.zeros( ( y.size ) )
   return v

def __pre__(y):
   """This is an empty preconditioner."""
   return y

def __bc__(y,t):
   """This is an empty boundary condition function. Leaves y untouched."""
   return y

class IMEX(PRK):
    '''
    This class is an implementation of implicit-explicit multistep schemes (IMEX)
    for the solution of:
    
    .. math::
    
       M\\frac{dy}{dt}=Ly+N(y,t)
    
    :param y: Initial condition
    :type y: A collection of :class:`sempy.Function`
    :param Time: An instance of the :class:`sempy.ode.Time` class.
    :kwargs: * **L** - Linear operator
             * **L_pre** - The FEM equivalent of L. Used to create 
               preconditioner. 
             * **force_function** - A function, possibly non-linear. 
               This is the function :math:`N(y,t)` in the equation above.
             * **bound_cond** - Strong BCs.
             * **imex_method** (*string*) - The IMEX method to be used. 
               The options are:
                    * First order forward/backward Euler: :literal:`euler`
                    * Crank-Nicolson/Adams-Bashforth: :literal:`cnab2`
                    * Modified Crank-Nicolson/Adams-Bashforth: 
                      :literal:`mcnab2`
                    * Second order BDF and second order extrapolatio: 
                      :literal:`bdfex2`
                    * Crank-Nicolson/leap-frog: :literal:`cnlf`
             * **init_steps** (*int*) - Number of time-steps used with a 
               first order method to kick-start the solution process. 
             * **linear_solver** - The linear solver. The default is 
               CG. If L is not SPD, then one can for example use bicgstab.
             * **iter_tol** (*float*)- Iterative tolerance.
    '''
    def __init__(self, y, Time, L = None, L_pre= None,
                  force_function = None, bound_cond = __bc__,
                  imex_method = 'euler', init_steps = 1,
                  file_increment = 1000,
                  hyper_filter = False,call_back=None,
                  linear_solver = None, iter_tol = 1e-10, maxiter = 1000):
        PRK.__init__( self, y, Time, L = L, L_pre= L_pre,
                      force_function = force_function, 
                      bound_cond = bound_cond,
                      file_increment = file_increment, 
                      hyper_filter = hyper_filter,call_back=call_back,
                      linear_solver = linear_solver, iter_tol = iter_tol,
                      maxiter=maxiter )
       
        self.init_steps = init_steps
        self.imex_method = imex_method

        if self.imex_method == 'euler':
            [self.alpha,self.beta,self.gamma,self.levels]=euler()
        if self.imex_method == 'bdfex2':
            [self.alpha,self.beta,self.gamma,self.levels]=bdfex2()
        if self.imex_method == 'bdfex3':
            [self.alpha,self.beta,self.gamma,self.levels]=bdfex3()
        if self.imex_method == 'mcnab2':
            [self.alpha,self.beta,self.gamma,self.levels]=mcnab2()
        if self.imex_method == 'cnab2':
            [self.alpha,self.beta,self.gamma,self.levels]=cnab2()
        if self.imex_method == 'cnlf':
            [self.alpha,self.beta,self.gamma,self.levels]=cnlf()
        if self.imex_method == 'cn':
            [self.alpha,self.beta,self.gamma,self.levels]=cn()
            
    def __rhs__(self,y):
        F = np.zeros((self.vector_size),float)
        s = np.zeros((self.vector_size),float)
        # Time derivative
        for i in range(0,self.alpha.size-1):   
            if abs(self.alpha[i])>0 :
                F = F - self.alpha[i] * (self.M * y[i])
        # Linear term
        for i in range(0,self.beta.size-1): 
            if abs(self.beta[i])>0 :
                s = self.L*y[i]
                F= F + self.Time.h*self.beta[i]*s
        # Explicit function
        for i in range(0,self.gamma.size):
            if abs(self.gamma[i])>0 :
                time = self.Time.time - \
                      (float(self.gamma.size)-1.0-float(i))*self.Time.h
                s = self.force_vec(y[i],time)
                F = F + self.Time.h*self.gamma[i]*s
         
        return F   
     
    def __initialize__(self,y_0,start,end):
        print 'initialize...'
        Time_local = sempy.ode.Time( self.Time.Space,start_time = start,
                             end_time = end,
                             time_steps = self.init_steps ) 
        euler= IMEX(y_0, Time_local, L = self.L_in, 
                    L_pre = self.L_pre,
                    force_function = self.force_function, 
                    bound_cond = self.bound_cond,
                    imex_method = 'euler', 
                    linear_solver = self.linear_solver, 
                    iter_tol = self.iter_tol,
                    maxiter = self.maxiter)
        y1 = euler.solve()
        del euler
        return y1
    
    def solve(self):
        '''
        Solves the ODE system
        '''
        #-- Solution vector y: y[0]=y^{n+1}, y[1]=y^{n}, y[2]=y^{n-1}, and so on
        y = np.zeros( (self.levels,self.vector_size), float )
        for i in range(self.levels):
            for j in range(self.number_of_eq):
                y[i] = self.component_to_vector( self.y0 )
        # Timesteps to file:
        k_file = range( self.file_increment, 
                        self.Time.time_steps+self.file_increment, 
                        self.file_increment )
        #--To file
        self.to_file(y[0])
        # -- Callback
        if not self.call_back == None:
            y_out = self.vector_to_component(y[0])  
            self.call_back(y_out ,self.Time.time,0)
        #-- Initialize
        factor=0.0
        for i in range(0,self.levels-2):
            self.Time.increment()
            factor = factor+1.0
            y_start = self.vector_to_component(y[i])
            y_temp = self.__initialize__( y_start,
                                          (factor-1.0)*self.Time.h,
                                          factor*self.Time.h )
            y[i+1] = self.component_to_vector( y_temp )
            # To file
            if (i+1) in k_file:
                self.to_file(y[i+1])
            # -- Callback
            if not self.call_back == None:
                y_out = self.vector_to_component(y[i+1])  
                self.call_back(y_out ,self.Time.time,i+1)
                
        #-- Time-stepping
        for i in range(self.levels-2,self.Time.time_steps):
            self.Time.increment()
            print '--> Time step number: ', i+1,'time=',self.Time.time
            # RHS
            b = self.__rhs__(y)
            # Boundary conditions
            y[self.levels-1]=self.apply_bc(y[self.levels-1],self.Time.time )
            # Helmholtz operator
            H = sempy.operators.MultipleOperators([self.M,self.L],
                      scaling_factor=[self.alpha[self.levels-1], 
                                    -self.Time.h*self.beta[self.levels-1]],
                      assemble='no').matrix
            # Solve linear system
            [y[self.levels-1],flag] = self.linear_solver.solve(
                                            H,b,y[self.levels-1])
            print 'iter=',flag
            # Update solution array
            for k in range(self.levels-1):
                y[k]=np.copy(y[k+1])
            # To file
            if (i+1) in k_file:
                self.to_file(y[self.levels-1])
            # -- Callback
            if not self.call_back == None:
                y_out = self.vector_to_component(y[self.levels-1])  
                self.call_back(y_out,self.Time.time,i+1)
        
        y_out = self.vector_to_component(y[self.levels-1])
        return y_out

class IMEX_old():
    """ 
    This is the imex multistep Solver class.
    ImpOp and ExpOp has to be square matrices or 
    LinearOperators as given by scipy.
    
       Usage
    
    :param A: Time instance.
    :param y0: Initial value. 
    :param Time: A L{blackbox.ode.basic.Time} object.
    :param ImpFunc: Function to be treated implicitly.
    :param ExpFunc: Function to be treated explicitly.
    :param ImpOp: Implicit operator.
    :param ExpOp: Explicit operator.
    :param BoundCond: Method/function with (possibly) time dependent boundary conditions.
    :param Mask:  Method/function depending on Dirichlet boundary conditions. 
    :param Method: The method can be L{imex.multistep.euler}, L{imex.multistep.cnab2}, L{imex.multistep.mcnab2}, 
           L{imex.multistep.cnlf}, L{imex.multistep.bdfex2} or a self defined one.
    :param InitSteps: For higher order methods one might need to start with small timesteps with the Euler method.
    :param LinearSolver: Sets the linear solver for system solves at each time-step. 
            Currently only: solvers.linear.cg.pcg.
    :param PreCond: Preconditioner passed to solvers.linear.cg.pcg. Has to approximately solve the linear system Hz=r.
    :param FileName: Sets the filename to store data. Every timestep solution is printed to file.
    """
    def __init__(self,y0,Time,
                 ImpFunc=__emptyfunc__,ExpFunc=__emptyfunc__,
                 ImpOp=[sp.array([0.0])],ExpOp=[sp.array([0.0])],
                 BoundCond=__bc__,Mask=__pre__,Method=euler,InitSteps=1,
                 LinearSolver=None, PreCond=__pre__,FileName=' '):
        
        self.Method       =Method 
        self.y0           =y0
        self.Time         =Time
        self.A            =ode.MultipleOperators(ImpOp,'implicit').MultiOperator# Implicit operator
        self.B            =ode.MultipleOperators(ExpOp,'explicit').MultiOperator# Explicit operator
        self.ImpFunc      =ImpFunc
        self.ExpFunc      =ExpFunc
        self.Mask         =Mask
        self.BoundCond    =BoundCond
        self.dof          =y0.size
        self.InitSteps    =InitSteps
        self.LinearSolver =LinearSolver
        self.PreCond      =PreCond
        self.FileName     =FileName
        [self.alpha,self.beta,self.gamma,self.levels]=Method()
        self.H            = ode.Helmholtz(self.A,self.Time.h,
                                      self.alpha[self.levels-1],
                                      self.beta[self.levels-1],
                                      self.dof,self.Mask).Matrix
    
    def __rhs__(self,y):
        F = np.zeros((self.dof))
        s = np.zeros((self.dof))
        for i in range(0,self.alpha.size-1):   # time derivative
            if abs(self.alpha[i])>0 :
                F = F - self.alpha[i] * y[i]
        if self.A:
            for i in range(0,self.beta.size-1): # implicit terms
                if abs(self.beta[i])>0 :
                    s = self.A*y[i]
                    F= F + self.Time.h*self.beta[i]*s
              
        for i in range(0,self.beta.size):      # implicit function
            if abs(self.beta[i])>0 :
                time = self.Time.time - \
                       (float(self.beta.size)-1.0-float(i))*self.Time.h
                s = self.ImpFunc(y[i],time)
                F = F + self.Time.h*self.beta[i]*s
         
        if self.B:
            for i in range(0,self.gamma.size):  # explicit terms
                if abs(self.gamma[i])>0 :
                    s = self.B*y[i]
                    F = F + self.Time.h*self.gamma[i]*s
            
        for i in range(0,self.gamma.size):     # explicit function
            if abs(self.gamma[i])>0 :
                time = self.Time.time - \
                      (float(self.gamma.size)-1.0-float(i))*self.Time.h
                s = self.ExpFunc(y[i],time)
                F = F + self.Time.h*self.gamma[i]*s
         
        #-- Mask boundaryconditions(F) upon exit
        return self.Mask(F)# compensate for Dirichlet conditions
     
    def __initialize__(self,y_0,start,end):
        print 'initialize...'
        t2 = ode.Time( start_time = start,
                       end_time = end,
                       time_steps = self.InitSteps ) 
        eul= Solver( y_0, t2, 
                     ImpFunc = self.ImpFunc,
                     ExpFunc = self.ExpFunc,
                     ImpOp = [self.A],ExpOp=[self.B],
                     BoundCond = self.BoundCond,
                     Mask = self.Mask,
                     Method = euler,
                     InitSteps = self.InitSteps,
                     LinearSolver = self.LinearSolver,
                     PreCond = self.PreCond )
        y1=eul.solve()
        print 'initialize stop. time=',t2.time
        return y1
    
    def solve(self):
        """ this solves the ODE problem."""
        #-- Solution vector y: y[0]=y^{n+1}, y[1]=y^{n}, y[2]=y^{n-1}, and so on
        y = np.zeros((self.levels,self.dof),float)
        for i in range(self.levels):
            y[i] = self.y0
        #--- Data to file
        FILE = file( self.FileName, 'w' )
        FILE.write( y[0].tostring() )
        #-- Initialize
        factor=0.0
        for i in range(0,self.levels-2):
            self.Time.increment()
            factor = factor+1.0
            y[i+1] = self.__initialize__( y[i],(factor-1.0)*self.Time.h,
                                        factor*self.Time.h )
            FILE.write(y[i+1].tostring())
        #-- Time-stepping
        for i in range(self.levels-2,self.Time.time_steps):
            self.Time.increment()
            print 'i=',i,'time=',self.Time.time
            b = self.__rhs__(y)
            if self.A: # System solve required
                y[self.levels-1]=self.BoundCond(
                                            y[self.levels-1],
                                            self.Time.time )
                [y[self.levels-1],flag]= self.LinearSolver(
                                            self.H,b,
                                            y[self.levels-1],
                                            precond = self.PreCond )
            else:     # System solve not required
                y[self.levels-1] = b / self.alpha[self.levels-1]
                y[self.levels-1] = self.BoundCond( y[self.levels-1],
                                                   self.Time.time )
        FILE.write(y[self.levels-1].tostring())

        for i in range(self.levels-1):# update solution array
          y[i]=y[i+1]
          
        return y[self.levels-1]
 
 
  




'''
Sempy: A Python implementation of the spectral element method.
Copyright (C) 2011,  Stian Jensen.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
''' 
import numpy as np
import sempy
import sempy.space.space_f90 as space_f90

import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import axes3d
import sempy.operators
import scipy.sparse.linalg as spl

import pysparse
from pysparse.direct import umfpack
# Compatibility with older versions
if int(pysparse.__version__[2]) >= 2:
    from pysparse.sparse.pysparseMatrix import PysparseMatrix #as pysparseMatrix
    from pysparse import direct
    
if int(pysparse.__version__[2]) <= 1:
    from pysparse.pysparseMatrix import PysparseMatrix
import time
import precond_f90



#print umfpack.factorize.__doc__


#import PyTrilinos.Epetra as Epetra
#import PyTrilinos.AztecOO as AztecOO
#import PyTrilinos.IFPACK as IFPACK

class SpaceFEM():
    '''
    Finite element space. 
    
    :param Space: A spectral element space
    :type Space: :class:`sempy.Space`
    
    **Example**::
    
      import sempy
      
      # SEM Space
      X = sempy.Space('square', n = 10, dim = 2)
      
      # FEM Space
      X_fem = sempy.precond.SpaceFEM( X )
      
      
    '''
    def __init__(self,Space):
        '''
        This is the init method.
        '''
        self.Space = Space
        self.type = 'GLL'
        
        self.n = 2
        self.dof = self.Space.dof
        self.basic = sempy.Basic()
        self.D = self.basic.derivative_matrix_gll( self.n )
        self.points, self.weights = \
                    self.basic.gauss_lobatto_legendre( self.n )
        
        if self.Space.dim == 1:
            self.noe = (self.Space.n-1)*self.Space.noe
            self.noe_bc = self.Space.noe_bc
            self.x, self.theta=self.fem_x()
            self.dim = 1
            self.jac_s = np.zeros((self.noe_bc), float)
            
        if self.Space.dim == 2:
            self.noe = (self.Space.n-1)*(self.Space.n-1)*self.Space.noe
            self.noe_bc = (self.Space.n-1)*self.Space.noe_bc
            self.x, self.y, self.theta = self.fem_xy()
            self.dim = 2
            self.jac_s = np.zeros((self.noe_bc,self.n), float)
        
        if self.Space.dim == 3:
            self.noe = (self.Space.n-1)*(self.Space.n-1)*\
                       (self.Space.n-1)*self.Space.noe
            self.noe_bc = (self.Space.n-1)*(self.Space.n-1)*\
                          self.Space.noe_bc
            self.x, self.y, self.z, self.theta = self.fem_xyz()
            self.dim = 3
            self.jac_s = np.zeros((self.noe_bc,self.n,self.n), float)
            
        self.theta_bc, self.physical_bc =  self.boundary_connectivity()
        self.jac = self._geometric()
        self.bc_type = self.Space.bc_type
        
        
    def _geometric(self):
        
        if self.Space.dim == 1:
            jac = space_f90.geometric_1d(self.x,self.D)
        if self.Space.dim == 2:
            jac = space_f90.geometric_2d(self.x,self.y,self.D)
        if self.Space.dim == 3:
            jac = space_f90.geometric_3d(self.x,self.y,self.z,self.D)
            
        return jac 
        
    def plot_mesh(self):
        '''
        Plots the numerical mesh.
        '''
        if self.dim == 1:
            u = np.zeros((self.noe,self.n),float)
            fig = plt.figure()
            for k in range(self.noe):
                plt.plot(self.x[k,:],u[k,:],'o-')
            plt.xlabel(r'$x$')
            plt.ylabel(r'$u$')
            plt.show()
            
        if self.dim == 2:
            u = np.zeros((self.noe,self.n,self.n),float)
            fig = plt.figure()
            ax = axes3d.Axes3D(fig)
            for k in range(self.noe):
                ax.plot_wireframe(self.x[k,:,:],self.y[k,:,:],u[k,:,:])
            plt.show()
    
    def plot_scalar(self,u,n_int=0):
        if n_int == 0:
            fig=plt.figure()
            ax=axes3d.Axes3D(fig)
            for k in range(self.noe):
                ax.plot_wireframe(self.x[k,:,:],self.y[k,:,:],u[k,:,:])
                #ax.plot_surface(self.x[k,:,:],self.y[k,:,:],u[k,:,:],rstride=1, cstride=1)
            ax.set_xlabel(r'$x$')
            ax.set_ylabel(r'$y$')
            ax.set_zlabel(r'$u$')
            plt.show()
        else:
            u_i = self.interpolation(u,n_int)
            x_i = self.interpolation(self.x, n_int)
            y_i = self.interpolation(self.y, n_int)
            
            fig = plt.figure()
            ax = axes3d.Axes3D(fig)
            for k in range(self.noe):
                #ax.plot_wireframe(x_i[k,:,:],y_i[k,:,:],u_i[k,:,:],rstride=1, cstride=1)
                ax.plot_surface(x_i[k,:,:],y_i[k,:,:],u_i[k,:,:],rstride=1, cstride=1)
            #ax.set_zlim3d(0, 1.0)
            ax.set_xlabel(r'$x$')
            ax.set_ylabel(r'$y$')
            ax.set_zlabel(r'$u$')
            plt.show()

    def l2_norm(self,u):
        '''
        Computes the :math:`L^2(\Omega)` - norm
           
        .. math::
           ||u||_{L^2(\\Omega)}=\\bigg(\\int_{\\Omega} u^2d\\Omega\\bigg)^{1/2}
              
        using GLL quadrature. 
           
        :param u: Basis coefficients in global data structure, i.e. 
                  :math:`u\\in\\mathbb R^{dof}` .
        :returns: The :math:`L^2` - norm as a floating number.
                   
        Usage::
           
             import sempy
             
             X = sempy.Space( filename = 'square', n = 5, dim = 2 ) 
             X_fem = sempy.SpaceFEM( X )      
             u = np.ones( ( X_fem.dof ), float )
             print 'L2-norm=',X_fem.l2_norm( u )
        '''
        if self.dim == 1:
            u_local = self.mapping_Q(u)
            norm = space_f90.l2norm_1d(self.weights,self.jac,u_local)
            
        if self.dim == 2:
            u_local = self.mapping_Q(u)
            norm = space_f90.l2norm_2d(self.weights,self.jac,u_local)
        
        if self.dim == 3:
            u_local = self.mapping_Q(u)
            print 'oobss l2 norm check mapping'
            norm = space_f90.l2norm_3d(self.weights,self.jac,u_local)
            
        return norm
    
    def quadrature(self,f):
        '''
        Computes the value :math:`\\kappa` of the integral 
        
        .. math::
           \\kappa = \\int_\\Omega f\\,d\\Omega 
           
        for :math:`f\\in L^2(\\Omega)` with GLL quadrature rules. 
        
        :param f: Basis coefficients of the function to be integrated 
                  given in local data representation 
                  (since :math:`f\\in L^2(\\Omega)`). 
        :returns: Value :math:`\\kappa` of the integral.
        
        Usage::
           
             import sempy
             X = sempy.Space( filename = 'square', n = 5, dim = 2 )
             X_fem = sempy.SpaceFEM( X )
    
             u = sempy.Function( X_fem, basis_coeff = X.x ) 
             kappa = X_fem.quadrature( u.basis_coeff )
        '''
        if self.dim == 1:
            kappa = space_f90.quadrature_1d(self.weights,self.jac,f)
        if self.dim == 2:
            kappa = space_f90.quadrature_2d(self.weights,self.jac,f)
        if self.dim == 3:
            kappa = space_f90.quadrature_3d(self.weights,self.jac,f)
        return kappa
        
    def theta_phys_bc(self, *args):
        '''
        :todo: element_edge
        '''
        
        if self.dim == 1:
            k=args[0]
            i       = self.theta_bc[k]
            phys    = self.physical_bc[k]
            #bc_type = self.Space.GTS.bc_type[phys-1]
            boundarycondtion_type = self.bc_type[phys-1]
            #element_edge = self.edge[k]
            element_edge=[0,0]
            
        if self.dim == 2:
            k=args[0]
            m=args[1]
            i       = self.theta_bc[k,m]
            phys    = self.physical_bc[k]
            #bc_type = self.Space.GTS.bc_type[phys-1]
            boundarycondtion_type = self.bc_type[phys-1]
            #element_edge = self.edge[k]
            element_edge=[0,0]
        
        if self.dim == 3:
            k=args[0]
            m=args[1]
            n=args[2]
            i       = self.theta_bc[k,m,n]
            phys    = self.physical_bc[k]
            #bc_type = self.Space.GTS.bc_type[phys-1]
            boundarycondtion_type = self.bc_type[phys-1]
            #element_edge = self.edge[k]
            element_edge=[0,0]
            
        return int(i), phys, boundarycondtion_type, element_edge

           
    def mapping_q(self,v):
        if self.dim == 1:
            w = space_f90.mapping_q_1d(v,self.theta)
        if self.dim == 2:
            w = space_f90.mapping_q_2d(v,self.theta)
        if self.dim == 3:
            w = space_f90.mapping_q_3d(v,self.theta)
        return w
        
    def mapping_qt(self,w):
        if self.dim == 1:
            v = space_f90.mapping_qt_1d(w,self.theta,self.dof)
        if self.dim == 2:
            v = space_f90.mapping_qt_2d(w,self.theta,self.dof)
        if self.dim == 3:
            v = space_f90.mapping_qt_3d(w,self.theta,self.dof)
        return v
    
    def local_to_global(self,w):
        if self.dim == 1:
            v = space_f90.local_to_global_1d(w,self.theta,self.dof)
        if self.dim == 2:
            v = space_f90.local_to_global_2d(w,self.theta,self.dof)
        if self.dim == 3:
            v = space_f90.local_to_global_3d(w,self.theta,self.dof)
        return v

    def mask(self,w):
        if self.dim == 1:
            for k in range(self.noe_bc):
                [i, name, bc_type, element_edge] = self.theta_phys_bc(k)
                if bc_type == 'Dir':
                    w[i] = 0.0
                        
        if self.dim == 2:
            for k in range(self.noe_bc):
                for m in range(self.n):
                    [i, name, bc_type, element_edge] = self.theta_phys_bc(k,m)
                    if bc_type == 'Dir':
                        w[i] = 0.0
        if self.dim == 3:
            for k in range(self.noe_bc):
                for m in range(self.n):
                    for n in range(self.n):
                        [i, name, bc_type, element_edge] = self.theta_phys_bc(k,m,n)
                        if bc_type == 'Dir':
                            w[i] = 0.0
        return w
        
    def mask_matrix(self,A):
        if self.dim == 1:
            for k in range(self.noe_bc):
                [i, phys, bc_type, element_edge]=self.theta_phys_bc(k)
                if bc_type == "Dir":
                    A[int(i),:] = 0.0
        if self.dim == 2:
            for k in range(self.noe_bc):
                for m in range(self.n):
                    [i, phys, bc_type, element_edge]=self.theta_phys_bc(k,m)
                    if bc_type == "Dir":
                        A[i,:] = 0.0
        if self.dim == 3:
            val = np.zeros((self.dof),float)
            row = np.zeros((self.dof),int)
            col = np.zeros((self.dof),int)
            col[:]=range(self.dof)
            for k in range(self.noe_bc):
                for m in range(self.n):
                    for n in range(self.n):
                        [i, phys, bc_type, element_edge]=self.theta_phys_bc(k,m,n)
                        if bc_type == "Dir":
                            #A[int(i),0:-1:1] = 0.0
                            row[:] = int(i)
                            A.put(val,row,col)
        return A
    
    def fem_x(self):
        x,theta = precond_f90.fem_coor_1d(self.Space.x,self.Space.theta, \
                                          self.n,self.noe)
        #x = np.zeros( (self.noe,self.n),float)
        #theta = np.zeros( (self.noe,self.n),int)
        
        #count=0
        #for k in range(self.Space.noe):
        #    for i in range(self.Space.n-1):
        #        x[count,0]=self.Space.x[k,i]
        #        x[count,1]=self.Space.x[k,i+1]
        #        theta[count,0]=self.Space.theta[k,i]
        #        theta[count,1]=self.Space.theta[k,i+1]
        #        count=count+1
        return x, theta
        
    def fem_xy(self):
        x,y,theta = precond_f90.fem_coor_2d( self.Space.x,
                                             self.Space.y,
                                             self.Space.theta, 
                                             self.n,self.noe )
        
        #x = np.zeros((self.noe,self.n,self.n),float)
        #y = np.zeros((self.noe,self.n,self.n),float)
        #theta = np.zeros((self.noe,self.n,self.n),int)
        
        #count=0
        #for k in range(self.Space.noe):
        #    for i in range(self.Space.n-1):
        #        for j in range(self.Space.n-1):
        #            x[count,0,0]=self.Space.x[k,i,j]
        #            x[count,1,0]=self.Space.x[k,i+1,j]
        #            x[count,0,1]=self.Space.x[k,i,j+1]
        #            x[count,1,1]=self.Space.x[k,i+1,j+1]
        #            y[count,0,0]=self.Space.y[k,i,j]
        #            y[count,1,0]=self.Space.y[k,i+1,j]
        #            y[count,0,1]=self.Space.y[k,i,j+1]
        #            y[count,1,1]=self.Space.y[k,i+1,j+1]
        #            theta[count,0,0]=self.Space.theta[k,i,j]
        #            theta[count,1,0]=self.Space.theta[k,i+1,j]
        #            theta[count,0,1]=self.Space.theta[k,i,j+1]
        #            theta[count,1,1]=self.Space.theta[k,i+1,j+1]
        #            count=count+1
        
        return x, y, theta
    
    def fem_xyz(self):
        #print precond_f90.fem_coor_3d.__doc__
        x,y,z,theta = precond_f90.fem_coor_3d( self.Space.x,
                                               self.Space.y,
                                               self.Space.z,
                                               self.Space.theta, 
                                               self.n,self.noe )
        
        #x = np.zeros((self.noe,self.n,self.n,self.n),float)
        #y = np.zeros((self.noe,self.n,self.n,self.n),float)
        #z = np.zeros((self.noe,self.n,self.n,self.n),float)
        #theta = np.zeros((self.noe,self.n,self.n,self.n),int)
        
        #count=0
        #for k in range(self.Space.noe):
        #    for m in range(self.Space.n-1):
        #        for n in range(self.Space.n-1):
        #            for o in range(self.Space.n-1):
        #                x[count,0,0,0]=self.Space.x[k,m,n,o]
        #                x[count,1,0,0]=self.Space.x[k,m+1,n,o]
        #                x[count,0,1,0]=self.Space.x[k,m,n+1,o]
        #                x[count,1,1,0]=self.Space.x[k,m+1,n+1,o]
        #                x[count,0,0,1]=self.Space.x[k,m,n,o+1]
        #                x[count,1,0,1]=self.Space.x[k,m+1,n,o+1]
        #                x[count,0,1,1]=self.Space.x[k,m,n+1,o+1]
        #                x[count,1,1,1]=self.Space.x[k,m+1,n+1,o+1]
                    
        #                y[count,0,0,0]=self.Space.y[k,m,n,o]
        #                y[count,1,0,0]=self.Space.y[k,m+1,n,o]
        #                y[count,0,1,0]=self.Space.y[k,m,n+1,o]
        #                y[count,1,1,0]=self.Space.y[k,m+1,n+1,o]
        #               y[count,0,0,1]=self.Space.y[k,m,n,o+1]
        #                y[count,1,0,1]=self.Space.y[k,m+1,n,o+1]
        #                y[count,0,1,1]=self.Space.y[k,m,n+1,o+1]
        #                y[count,1,1,1]=self.Space.y[k,m+1,n+1,o+1]
                        
        #                z[count,0,0,0]=self.Space.z[k,m,n,o]
        #                z[count,1,0,0]=self.Space.z[k,m+1,n,o]
        #                z[count,0,1,0]=self.Space.z[k,m,n+1,o]
        #                z[count,1,1,0]=self.Space.z[k,m+1,n+1,o]
        #                z[count,0,0,1]=self.Space.z[k,m,n,o+1]
        #                z[count,1,0,1]=self.Space.z[k,m+1,n,o+1]
        #                z[count,0,1,1]=self.Space.z[k,m,n+1,o+1]
        #                z[count,1,1,1]=self.Space.z[k,m+1,n+1,o+1]
                        
        #                theta[count,0,0,0]=self.Space.theta[k,m,n,o]
        #                theta[count,1,0,0]=self.Space.theta[k,m+1,n,o]
        #                theta[count,0,1,0]=self.Space.theta[k,m,n+1,o]
        #                theta[count,1,1,0]=self.Space.theta[k,m+1,n+1,o]
        #                theta[count,0,0,1]=self.Space.theta[k,m,n,o+1]
        #                theta[count,1,0,1]=self.Space.theta[k,m+1,n,o+1]
        #                theta[count,0,1,1]=self.Space.theta[k,m,n+1,o+1]
        #                theta[count,1,1,1]=self.Space.theta[k,m+1,n+1,o+1]
                        
        #                count=count+1
        
        return x, y, z, theta
        
    def boundary_connectivity(self):
        
        if self.Space.dim == 1:
            theta_bc    = np.zeros((self.noe_bc),int)
            physical_bc = np.zeros((self.noe_bc),int)
                        
            count = 0
            for k in range(self.Space.noe_bc):
                [i, phys, bc_type_gll, element_edge] = self.Space.theta_phys_bc(k)
                theta_bc[count] = self.Space.theta_bc[k]
                physical_bc[count] = phys
                count = count+1
            
        if self.Space.dim == 2:
            theta_bc    = np.zeros((self.noe_bc,self.n),int)
            physical_bc = np.zeros((self.noe_bc),int)
            #edge        = np.zeros((self.noe_bc, 2), int)
        
            count = 0
            for k in range(self.Space.noe_bc):
                for m in range(self.Space.n-1):
                    [i, phys, bc_type_gll, element_edge] = self.Space.theta_phys_bc(k,m)
                    theta_bc[count,0] = self.Space.theta_bc[k,m]
                    theta_bc[count,1] = self.Space.theta_bc[k,m+1]
                    physical_bc[count] = phys
                    #bc_type_fem.append(bc_type_gll)
                    #edge[count,0] = # internal element number
                    #edge[count,1] = element_edge[1]# edge
                    #if element_edge[1] == 0:
                    #    k_fem=element_edge[1]*
                    # k_fem=theta[k,m,0]
                    count = count+1
        
        if self.Space.dim == 3:
            theta_bc    = np.zeros((self.noe_bc,self.n,self.n),int)
            physical_bc = np.zeros((self.noe_bc),int)
            
            count = 0
            for k in range(self.Space.noe_bc):
                for m in range(self.Space.n-1):
                    for n in range(self.Space.n-1):
                        [i, phys, bc_type_gll, element_edge] = self.Space.theta_phys_bc(k,m,n)
                        theta_bc[count,0,0] = self.Space.theta_bc[k,m,n]
                        theta_bc[count,1,0] = self.Space.theta_bc[k,m+1,n]
                        theta_bc[count,0,1] = self.Space.theta_bc[k,m,n+1]
                        theta_bc[count,1,1] = self.Space.theta_bc[k,m+1,n+1]
                        
                        physical_bc[count] = phys
                        count = count+1
            
        return theta_bc, physical_bc
        
    def sem_to_fem(self,u_sem):
        '''
        A mapping from the SEM space to FEM space. 
        '''
        if self.dim == 1:
            u_fem = precond_f90.sem_to_fem_1d(u_sem,self.n,self.noe)
            #u_fem = np.zeros((self.noe,self.n),float)
            #count=0
            #for k in range(self.Space.noe):
            #    for i in range(self.Space.n-1):
            #        u_fem[count,0]=u_sem[k,i]
            #        u_fem[count,1]=u_sem[k,i+1]
            #        count=count+1
                        
        if self.dim == 2:
            u_fem = precond_f90.sem_to_fem_2d(u_sem,self.n,self.noe)
            #u_fem = np.zeros((self.noe,self.n,self.n),float)
            #count=0
            #for k in range(self.Space.noe):
            #    for i in range(self.Space.n-1):
            #        for j in range(self.Space.n-1):
            #            u_fem[count,0,0]=u_sem[k,i,j]
            #            u_fem[count,1,0]=u_sem[k,i+1,j]
            #            u_fem[count,0,1]=u_sem[k,i,j+1]
            #            u_fem[count,1,1]=u_sem[k,i+1,j+1]
            #            count=count+1
        if self.dim == 3:
            u_fem = precond_f90.sem_to_fem_3d(u_sem,self.n,self.noe)
            
        return u_fem
                                          
                                  

        
class Preconditioner:
    '''
    Preconditioner. 
        
    :param Space: Space
    :type Space: :class:`sempy.Space`
    :param Operators: A collection of operators
    :param scaling_factor: Scaling factors for operators in 
                           :literal:`Operators`.
    :param drop_tol: Drop tolerance for ILU
    :param string library: Either :literal:`superlu` or :literal:`umfpack`. 
    
    :attributes: * **matrix** - The preconditioner. A matrix that approximates the inverse of the SEM matrix.
    
    **Example**::
    
      import sempy
      
      X = sempy.Space( filename = 'square', n = 30, dim = 2 ) 
      laplace = sempy.operators.Laplacian( X, fem_assemble = 'yes' )
      
      A_fem = laplace.matrix_fem

      pre = sempy.precond.Preconditioner( X, [A_fem] , drop_tol = 0.5 )
      P = pre.matrix

     
    '''
    def __init__( self, Space, Operators, 
                  scaling_factor = 'none', drop_tol = 0.0,
                  library = 'superlu',silent = False ):
        
        
        self.Space = Space
        self.library = library
        self.silent = silent
        if scaling_factor=='none':
            self.scaling_factor = np.ones((len(Operators)),float)
        else:
            self.scaling_factor = scaling_factor
        self.drop_tol = drop_tol
        Q = sempy.operators.MultipleOperators(
                            Operators,
                            scaling_factor = self.scaling_factor,
                            assemble = 'yes',silent=self.silent ).matrix
        Q = self.__convert_to_ll_mat__(Q)
        if not self.silent:
            print ''
            print '**** Factorizing, pre         ****'
        
        if pysparse.__version__[2] >='2':
            if self.library == 'superlu':
                self.LU = direct.superlu.factorize( Q.to_csr(),
                                           drop_tol=self.drop_tol )
            if self.library == 'umfpack':
                self.LU = umfpack.factorize( Q )
                
        if pysparse.__version__[2] <='1':
            self.LU = pysparse.superlu.factorize(Q.to_csr(),
                                                 drop_tol=self.drop_tol)
        if not self.silent:
            print '**** Non-zero entries=',self.LU.nnz
        del Q
        self.matrix = spl.LinearOperator( 
                                (self.Space.dof,self.Space.dof),
                                matvec = self.__action__, 
                                dtype = 'float' )
    
    def __convert_to_ll_mat__(self,A):
        Q = pysparse.spmatrix.ll_mat( self.Space.dof, self.Space.dof )
                
        val,irow,jcol = A.find()
        Q.update_add_at(val,irow,jcol)
        del val,irow,jcol
        
        if self.Space.dim == 1:
            for k in range(self.Space.noe_bc):
                [i, phys, bc_type, element_edge] = \
                                        self.Space.theta_phys_bc(k)
                if bc_type == "Dir":
                    Q[int(i),int(i)] = 1.0
                    
        if self.Space.dim == 2:
            for k in range(self.Space.noe_bc):
                for m in range(self.Space.n):
                    [i, phys, bc_type, element_edge] = \
                                        self.Space.theta_phys_bc(k,m)
                    if bc_type == "Dir":
                        Q[int(i),int(i)] = 1.0
                        
        if self.Space.dim == 3:
            for k in range(self.Space.noe_bc):
                for m in range(self.Space.n):
                    for n in range(self.Space.n):
                        [i, phys, bc_type, element_edge] = \
                                        self.Space.theta_phys_bc(k,m,n)
                        if bc_type == "Dir":
                            Q[int(i),int(i)] = 1.0
                        
        return Q
        
    def __action__(self,v):
        w = np.zeros(self.Space.dof, float)
        self.LU.solve(v,w)
        return w

                           
#class PressurePrecond:
#    def __init__(self,Space,SpaceGL):
#        self.Space = Space
#        self.SpaceGL = SpaceGL
#        
#        # Laplacian
#        self.lap = sempy.operators.Laplacian( self.Space, 
#                                              mask = 'no',
#                                              fem_assemble = 'yes' )
#        #self.A = self.lap.matrix
#        self.A = self.lap.matrix_fem
#        self.A_pre = sempy.precond.Preconditioner( self.Space, 
#                                                   [self.A], 
#                                                   library = 'superlu',
#                                                   drop_tol = 1.0 ).matrix
#        self.poisson_solver = sempy.linsolvers.Krylov(
#                                                tol = 1.0e-16,
#                                                solver_type = 'cg')
#        #self.matrix =
#        self.x = np.zeros((self.Space.dof),float)
#        self.matrix = spl.LinearOperator( 
#                          ( self.Space.dof,self.Space.dof ),
#                            matvec = self.__action_pre__,
#                            dtype = 'float' )
#        
#    def __action_pre__(self,v):
#        t1=time.time()
#        self.x[:] = 0.0
#        #[x,flag] = self.poisson_solver.solve(self.A,v,self.x)
#        x=self.A_pre*v
#        t2=time.time()
#        print 'pre. ex time=',t2-t1#,'iter=',flag
#        
#        return x
        
 





if __name__ == "__main__":
    #import doctest
    #doctest.testmod()
    import sempy
    
    X = sempy.Space(filename = 'cube', n = 4, dim = 3)
    X_fem = sempy.precond.SpaceFEM(X)
    #X_fem.plot_mesh()
    #A = sempy.operators.Laplacian(X).matrix
    
    #f = sempy.Function( X, basis_coeff = 1.0 )
    #b = sempy.operators.Mass( X ).action_local( f.basis_coeff )
    #u = sempy.Function( X, basis_coeff = 0.0 )
    
    #[v, flag] = sempy.linsolvers.Krylov().solve(A, b, u.glob())
    #u.basis_coeff = X.mapping_q(v)
    #u.plot_wire()
    



'''
Sempy: A Python implementation of the spectral element method.
Copyright (C) 2011,  Stian Jensen.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
''' 

import numpy as np
import sempy
import sempy.space.space_f90 as space_f90

import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import axes3d
from matplotlib import cm

import time

import sys, os 

def MeshConvert2D(filename,elem_dof):
    '''
    Takes as input a gmsh file (filename) and gmsh element type (elem_dof).
    Returns: x, y, physical_names, theta, physical, theta_bc, physical_bc, dof, noe
    '''
    #print 'Reading gmsh file....'
    #--- Physical names
    f=open(filename,'r')
    i=0
    for line in f:
        i=i+1
        if line == "$PhysicalNames\n":
            name_start=i+1
        if line == "$EndPhysicalNames\n":
            name_end=i
    f.close()
    
    f=open(filename,'r')
    f_names=open('names.txt','w')
    i=0
    for line in f:
        i=i+1
        if i > name_start and i < name_end:
            f_names.write(line)
    f.close()
    f_names.close()
    f_names=open('names.txt','r')
    physical_names = np.genfromtxt(f_names,usecols=(0,1),dtype=int)
    f_names.close()
    #print 'physical_names='
    #print physical_names
    #print '**************'
    # Determining the bc type, i.e. "dir" (strong, Dirichlet) or "Nat" 
    # (natural like Robin and Neumann)
    f_names=open('names.txt','r')
    bc_type=['0']
    for line in f_names:
        #print line
        if int(line[0]) == 1:
            bc_type.append(line[5:8])
    f_names.close()
    bc_type.remove('0')
    #print 'bc_type=',bc_type
    #print 'physical_names=',physical_names
    #print '**************'
    
    #--- Locating the nodes and elements in the  gmsh-file:
    f=open(filename,'r')
    i=0
    for line in f:
        i=i+1
        if line == "$Nodes\n":
            nodes_start=i+1
        if line == "$EndNodes\n":
            nodes_end=i        
        if line == "$Elements\n":
            elements_start=i+1
        if line == "$EndElements\n":
            elements_end=i
    f.close()

    #--- Creating temporary files of nodes and elements
    f=open(filename,'r')
    f_nodes=open('nodes.txt','w')
    f_elements=open('elements.txt','w')

    i=0
    for line in f:
        i=i+1
        if i > nodes_start and i < nodes_end:
            f_nodes.write(line)
        if i > elements_start and i < elements_end:
            f_elements.write(line)
    f.close()
    f_nodes.close()
    f_elements.close()

    #--- Nodal points (x,y)
    f_nodes = open('nodes.txt','r')
    A = np.genfromtxt(f_nodes,dtype=float)
    f_nodes.close()

    x = A[:,1]
    y = A[:,2]
    #print 'A=',A

    #--- Connectivity array (theta)
    # Finding where to cut off boundary elements from internal
    #
        
    if elem_dof == 3:
        f_elements = open('elements.txt','r')
        A = np.genfromtxt(f_elements, dtype=int,
                      usecols=(0,1,2,3,4,5,6,7,8))
        f_elements.close()

    # Removing the boundary elements
    if elem_dof == 2:
        element_type = 3 # gmsh type
    if elem_dof == 3:
        element_type = 10 # gmsh type
    for i in range(A.shape[0]):
        if A[i,1] == element_type:
            cutoff=i
            break
    # Boundary
    
    # Boundary elements
    theta_bc = np.zeros( ( cutoff, elem_dof ), int )
    physical_bc = np.zeros( ( cutoff ), int )
    for i in range(cutoff):
        theta_bc[i,0]=A[i,6]-1
        theta_bc[i,2]=A[i,7]-1
        theta_bc[i,1]=A[i,8]-1
        physical_bc[i]=A[i,3]
    #print 'physical_bc='
    #print physical_bc
    # Creating array of elements without boundary elements
    f_elements = open('elements.txt','r')
    #internal_elements = np.genfromtxt(f_elements, dtype=int, skiprows=cutoff)
    
    
    # Just some stuff to comply  with different versions of numpy.
    ver = np.fromstring( np.__version__, dtype = float, sep = '.' )
    if ver[0] > 1.3:
        internal_elements = np.genfromtxt( f_elements, dtype = int, 
                                           skip_header = cutoff )
    else:
        internal_elements = np.genfromtxt( f_elements, dtype = int, 
                                           skiprows = cutoff )
    f_elements.close()


    k = internal_elements.shape[0]
    if elem_dof == 2:
        theta        = np.zeros((k,2,2),int)
        physical     = np.zeros((k),int)
        theta[:,0,0] = internal_elements[:,6]-1
        theta[:,1,0] = internal_elements[:,7]-1
        theta[:,1,1] = internal_elements[:,8]-1
        theta[:,0,1] = internal_elements[:,9]-1
        physical[:]  = internal_elements[:,3]
    
    if elem_dof == 3:
        theta        = np.zeros((k,3,3),int)
        physical     = np.zeros((k),int)
        theta[:,0,0] = internal_elements[:,6]-1
        theta[:,1,0] = internal_elements[:,10]-1
        theta[:,2,0] = internal_elements[:,7]-1
        theta[:,2,1] = internal_elements[:,11]-1
        theta[:,1,2] = internal_elements[:,12]-1
        theta[:,0,1] = internal_elements[:,13]-1
        theta[:,1,1] = internal_elements[:,14]-1
        theta[:,2,2] = internal_elements[:,8]-1
        theta[:,0,2] = internal_elements[:,9]-1
        physical[:]  = internal_elements[:,3]

    dof=x.shape[0]
    noe=theta.shape[0]
    #print ' number of elements=',noe
    #print ' degrees of freedom=',dof
    #print '... finished reading file.'
    return x, y, physical_names, theta, physical, theta_bc, physical_bc, dof, noe, bc_type



class GmeshToSpectral2D():
    def __init__(self,filename,n):
        #print 'Creating Gmsh object ..'
        self.basic = sempy.Basic()
        self.n = n
        self.D = self.basic.derivative_matrix_gll( self.n )
        x_glob, y_glob, self.physical_names, self.theta, self.physical,\
        self.theta_bc, self.physical_bc, self.dof, self.noe, self.bc_type\
            = MeshConvert2D(filename,self.n)
        self.points, self.weights = self.basic.gauss_lobatto_legendre(self.n)
        self.x=self.mapping_q(x_glob)
        self.y=self.mapping_q(y_glob)
        #print '... finished creating Gmsh object.'
        
    def theta_phys_bc(self,k,m):
        i=self.theta_bc[k,m]
        phys=self.physical_bc[k]
        return i,phys
        
    def theta_phys(self,k,m,n):
        i=self.theta[k,m,n]
        phys=self.physical[k]
        return i,phys
        
    def mapping_q(self,v):
        # from global to local
        #w = np.zeros((self.noe,self.n,self.n),float)
        #for k in range(self.noe):
        #    for m in range(self.n):
        #        for n in range(self.n):
        #            i=self.theta[k,m,n]
        #            w[k,m,n]=v[i]
        w=space_f90.mapping_q_2d(v,self.theta)
        return w
        
    def mapping_qt(self,w):
        #v = np.zeros(self.dof,float)
        #for k in range(self.noe):
        #    for m in range(self.n):
        #        for n in range(self.n):
        #            i=self.theta[k,m,n]
        #            v[i]=v[i]+w[k,m,n]
        v=space_f90.mapping_qt_2d(w,self.theta,self.dof)
        return v

  
        
class MeshObject2D():
    '''
    This class creates a sem mesh object based on a gmsh file and the chosen polynomial degree. 
    
    :param filename: A gmsh file with format .msh
    :param n: Number of nodal points in each direction of an element. Hence, the polynomial 
              degree of the basis functions is :math:`N=n-1`.
    :param n_gmsh (optional): Gmsh dof. n_gmsh=2 or n_gmsh=3
    
    :attributes: 
                      
                 * *points* - GLL quadrature points 
                   :math:`\\underline{\\xi}\\in\\mathbb R^{N+1},\\ -1\\le \\xi_j\\le 1`
                 * *weights* - GLL quadrature weights :math:`\\underline{\\rho}\\in\\mathbb R^{N+1}` 
                   corresponding to the points :math:`\\underline{\\xi}`. 
                   Used to evaluate an integral over the reference domain:
                 
                   .. math::
                      \\hat\\Omega\\in\\mathbb R^1: \\int_{\\hat\\Omega} f d\\hat\\Omega\\approx
                      \\sum_i\\rho_if_i
                      
                      \\hat\\Omega\\in\\mathbb R^2: \\int_{\\hat\\Omega} f d\\hat\\Omega\\approx
                      \\sum_i\\sum_j\\rho_i\\rho_jf_{ij}
                      
                 * *D* - GLL derivative matrix 
                   :math:`\\underline D\\in\\mathbb R^{(N+1)\\times (N+1)}`. For a 1D problem:
    
                   .. math::
                      u_N'(\\xi_j)=\\sum_i l_i'(\\xi_j) u_i=\\sum_i D_{ji}u_i\\Rightarrow
                      D_{ji}=l_i'(\\xi_j)
                 * *noe* - Number of elements
                 * *noe_bc* - Number of boundary elements
                 * *x* - Coordinate array (given in local data distribution). 
                 * *y* -
                 * *jac* - The Jacobian :math:`|\\underline{J}|`.
                 * *jac_s* - The surface Jacobian :math:`|\\underline{J}^s|`. jac_s[noe_bc,n]. 
                 * *dof* - Global degrees of freedom
    
    Usage::
    
       import sempy
      
       filename = 'square.msh'
       n = 6
       mesh = MeshObject2D(filename,n)
       mesh.plot()
    
    '''
    def __init__(self,filename,n,n_gmsh=3):
        '''
        This is the init method.
        '''
        print ''
        print '**** Creating Space object     ****'
        self.GTS = GmeshToSpectral2D(filename,n_gmsh)
        self.bc_type = self.GTS.bc_type
        self.basic = sempy.Basic()
        
        self.dim=2
        
        self.physical = self.GTS.physical
        self.n = n
        self.D = self.basic.derivative_matrix_gll(self.n)
        self.points, self.weights = self.basic.gauss_lobatto_legendre(self.n)
        self.noe = self.GTS.noe
        
        self.x = self.interpolation(self.GTS.x,self.n)
        #self.x = self.local_to_global(self.x)
        #self.x = self.mapping_q(self.x)
        
        self.y = self.interpolation(self.GTS.y,self.n)
        #self.y = self.local_to_global(self.y)
        #self.y = self.mapping_q(self.y)
        
        #print ' connectivity'
        t1 = time.clock()
        self.theta, self.dof = self.connectivity()
        t2 = time.clock()
        #print ' ... connectivity finished. ex time=',t2-t1
        #print ' boundary connectivity'
        self.theta_bc, self.physical_bc, self.jac_s, self.edge = self.boundary_connectivity()
        #print ' ... boundary connectivity finished.'
        self.noe_bc = self.GTS.theta_bc.shape[0]
        self.jac = self.geometric()
        
        self.n_x, self.n_y = self.surface_unit_normal()
        
        print '**** dim =',self.dim
        print '**** noe =',self.noe
        print '**** dof =',self.dof
        print '****   n =',self.n
        print '**** Finished creating instance.' 
        #print '**** Creating Mesh object   ****'
        
    def geometric(self):
        # Jacobian
        #jac   = np.zeros((self.noe,self.n,self.n),float)
        #x_xi  = np.zeros((self.noe,self.n,self.n),float)
        #x_eta = np.zeros((self.noe,self.n,self.n),float)
        #y_xi  = np.zeros((self.noe,self.n,self.n),float)
        #y_eta = np.zeros((self.noe,self.n,self.n),float)
        #for k in range(self.noe):
        #    for alpha in range(self.n):
        #        for beta in range(self.n):
        #            for i in range(self.n):
        #                x_xi[k,alpha,beta]  = x_xi[k,alpha,beta]  + self.D[alpha,i] * self.x[k,i,beta]
        #                y_xi[k,alpha,beta]  = y_xi[k,alpha,beta]  + self.D[alpha,i] * self.y[k,i,beta]
        #                y_eta[k,alpha,beta] = y_eta[k,alpha,beta] + self.D[beta,i]  * self.y[k,alpha,i]
        #                x_eta[k,alpha,beta] = x_eta[k,alpha,beta] + self.D[beta,i]  * self.x[k,alpha,i]
        #jac = x_xi * y_eta - x_eta * y_xi
        jac = space_f90.geometric_2d(self.x,self.y,self.D)
        return jac
        
    def plot_mesh(self):
        u = np.zeros((self.noe,self.n,self.n),float)
        
        fig = plt.figure()
        ax = axes3d.Axes3D(fig)
        for k in range(self.noe):
            ax.plot_wireframe(self.x[k,:,:],self.y[k,:,:],u[k,:,:])
        
        natural=['Neu','rob']
        for k in range(self.noe_bc):
            i, phys, bc_type, edge=self.theta_phys_bc(k,0)
            if edge[1]==0:
                k_e=edge[0]
                if bc_type == "Dir":
                #if bc_type in natural:
                    ax.plot(self.x[k_e,:,0],self.y[k_e,:,0],u[k_e,:,0],'ro')
                else:
                    ax.plot(self.x[k_e,:,0],self.y[k_e,:,0],u[k_e,:,0],'go')
            if edge[1]==1:
                k_e=edge[0]
                if bc_type == "Dir":
                #if bc_type in natural:
                    ax.plot(self.x[k_e,:,self.n-1],self.y[k_e,:,self.n-1],u[k_e,:,self.n-1],'ro')
                else:
                    ax.plot(self.x[k_e,:,self.n-1],self.y[k_e,:,self.n-1],u[k_e,:,self.n-1],'go')
            if edge[1]==2:
                k_e=edge[0]
                if bc_type == "Dir":
                #if bc_type in natural:
                    ax.plot(self.x[k_e,0,:],self.y[k_e,0,:],u[k_e,0,:],'ro')
                else:
                    ax.plot(self.x[k_e,0,:],self.y[k_e,0,:],u[k_e,0,:],'go')
            if edge[1]==3:
                k_e=edge[0]
                if bc_type == "Dir":
                #if bc_type in natural:
                    ax.plot(self.x[k_e,self.n-1,:],self.y[k_e,self.n-1,:],u[k_e,self.n-1,:],'ro')
                else:
                    ax.plot(self.x[k_e,self.n-1,:],self.y[k_e,self.n-1,:],u[k_e,self.n-1,:],'go')
                
        #plt.legend((label))#'Dirichlet')
        ax.set_xlabel(r'$x$')
        ax.set_ylabel(r'$y$')
        plt.show()
    
    def plot_scalar(self,u,n_int=0):
        if n_int == 0:
            fig=plt.figure()
            ax=axes3d.Axes3D(fig)
            for k in range(self.noe):
                ax.plot_wireframe(self.x[k,:,:],self.y[k,:,:],u[k,:,:])
                #ax.plot_surface(self.x[k,:,:],self.y[k,:,:],u[k,:,:],rstride=1, cstride=1,cmap=cm.jet)
            ax.set_xlabel(r'$x$')
            ax.set_ylabel(r'$y$')
            ax.set_zlabel(r'$u$')
            plt.show()
        else:
            u_i = self.interpolation(u,n_int)
            x_i = self.interpolation(self.x, n_int)
            y_i = self.interpolation(self.y, n_int)
            
            fig = plt.figure()
            ax = axes3d.Axes3D(fig)
            for k in range(self.noe):
                #ax.plot_wireframe(x_i[k,:,:],y_i[k,:,:],u_i[k,:,:],rstride=1, cstride=1)
                #ax.plot_surface(x_i[k,:,:],y_i[k,:,:],u_i[k,:,:],rstride=self.n, cstride=self.n)
                ax.plot_surface(self.x[k,:,:],self.y[k,:,:],u[k,:,:],rstride=1, cstride=1,cmap=cm.jet)
            #ax.set_zlim3d(0, 1.0)
            ax.set_xlabel(r'$x$')
            ax.set_ylabel(r'$y$')
            ax.set_zlabel(r'$u$')
            plt.show()
            
    def plot_basis_function(self,n_g,show=True,font_size=20):
            n_int = 25
            u_glob = np.zeros((self.dof),float)
            u_glob[n_g] = 1.0
            u = self.mapping_q(u_glob)
            
            u_i = self.interpolation(u,n_int)
            x_i = self.interpolation(self.x, n_int)
            y_i = self.interpolation(self.y, n_int)
            
            fig_number = np.random.randint(0,101)
            fig = plt.figure(fig_number)
            levels=np.arange(-0.2,1.01,0.02)
            for k in range(self.noe):
                plt.contourf(x_i[k,:,:],y_i[k,:,:],u_i[k,:,:],levels,\
                             cmap=cm.get_cmap('jet',len(levels)-1))
            plt.xlabel(r'$x$', fontsize = font_size )
            plt.ylabel(r'$y$', fontsize = font_size )
            plt.axis([self.x.min(),self.x.max(),
                      self.y.min(),self.y.max()])
            plt.axes().set_aspect('equal')
            ax = plt.gca()
            for label in ax.xaxis.get_ticklabels():
                label.set_fontsize( font_size )
            for label in ax.yaxis.get_ticklabels():
                label.set_fontsize( font_size )
            cbar = plt.colorbar()
            for t in cbar.ax.get_yticklabels():
                t.set_fontsize(font_size)
            if show:
                plt.show()
            
        
    def l2_norm(self,u):
        """Computes the :math:`L^2` - norm
           
           .. math::
              ||u||_{L^2(\\Omega)}=\\bigg(\\int_{\\Omega} u^2d\\Omega\\bigg)^{1/2}
              
           using GLL quadrature. 
           
           :param u: Basis coefficients in global data structure, i.e. :math:`u\\in\\mathbb R^{dof}` .
           :returns: The :math:`L^2` - norm as a floating number.
           :todo: Fortran code. 
           
           Usage::
           
             u  = np.ones((mesh.dof),float)
             print 'L2-norm=',mesh.l2norm(u)

        """
        #u_local = self.mapping_q(u)
        #norm = 0.0 
        #for k in range(self.noe):
        #    for i in range(self.n):
        #        for j in range(self.n):
        #            norm = norm + self.weights[i] * self.weights[j] * self.jac[k,i,j] * \
        #                   u_local[k,i,j]*u_local[k,i,j]
        #norm=np.sqrt(norm)
        norm = space_f90.l2norm_2d(self.weights,self.jac,u)
        return norm
    
    def quadrature(self,f):
        """Computes the value of the integral 
        
           .. math::
              \\kappa = \\int_\\Omega f\\,d\\Omega 
           
           for :math:`f\\in L^2(\\Omega)` with GLL quadrature rules. 
        
           :param f: Basis coefficients of the function to be integrated given in local
                     data representation (since f in L2). 
           :returns: Value :math:`\\kappa` of the integral.
           :todo: Fortran code. 
        
        """
        #kappa = 0.0
        #for k in range(self.noe):
        #    for i in range(self.n):
        #        for j in range(self.n):
        #            kappa = kappa + self.weights[i] * self.weights[j] *\
        #                            self.jac[k,i,j] * f[k,i,j]
        kappa = space_f90.quadrature_2d(self.weights,self.jac,f)
        return kappa
                    
        
    def theta_phys(self, k, m, n):
        '''Local to global connectivity method. Takes as input the local node 
           description and returns the global node number and the physical name 
           of the element. 

           :param k: Element number
           :param m: Node number in the xi direction
           :param n: Node number in the eta direction
           :type k: integer
           :type m: integer
           :type n: integer
           :returns: An integer tuple of global node number and corresponding physical name.         
           
             
           Example::
              
              for k in range(mesh.noe):
                  for m in range(mesh.n):
                      for n in range(mesh.n):
                          i, phys = mesh.theta_phys(k,m,n)
        '''
        i    = self.theta[k,m,n]
        phys = self.physical[k]
        return i, phys

    def theta_phys_bc(self, k, m):
        '''
           Local to global connectivity method for boundary points. 
           
           :param k: Element number
           :param m: Node number along the boundary element
           :type k: integer
           :type m: integer
           :returns: A tuple [i, name, bc_type, element_edge] of global node number i, corresponding 
                     physical name and type of boundary condition, either Dir or Nat. The type 
                     has to be specified in the Gmsh .geo file. 
                     element_edge: gives the internal element number and edge.
           
           
           To use this to apply boundary conditions to u::
           
              u_1 = 10.0
              u_2 =  5.0
              for k in range(mesh.noe_bc):              # Loops over the boundary elements
                  for m in range(mesh.n):               # Loops over the points along the boundary element
                      [i, name, bc_type, edge] = mesh.theta_phys_bc(k,m)
                      if name == 1:
                          u[i] = u_1                    # Assign bc
                      if name == 2:
                          u[i] = u_2                    # Assing bc 
           
        '''
        i       = self.theta_bc[k,m]
        phys    = self.physical_bc[k]
        boundarycondtion_type = self.bc_type[phys-1]
        element_edge = self.edge[k]
        
        return i, phys, boundarycondtion_type, element_edge
    
    def gradient_vector(self,u):
        u_x,u_y = space_f90.gradient_vector_2d(u,self.x,self.y,self.jac,self.D)
        return u_x,u_y
    
    
    def surface_flux(self,u_grad):
        u_bx = self.boundary_value(u_grad[0])
        u_by = self.boundary_value(u_grad[1])
        #mu_b = self.boundary_value(mu)
        
        kappa = 0.0
        for k in range(self.noe_bc):
            for m in range(self.n):
    #            [i, phys, boundarycondtion_type, element_edge] = self.theta_phys_bc(k,m)
                 u_n = u_bx[k,m] * self.n_x[k,m] + u_by[k,m] * self.n_y[k,m]
                 #coeff = (self.n_x[k,m] +self.n_y[k,m])/(self.n_x[k,m]*self.n_y[k,m])
                 #print coeff
                 #u_n = u_bx[k,m] * self.n_x[k,m]*self.n_x[k,m] + \
                 #      u_by[k,m] * self.n_y[k,m]*self.n_y[k,m]
                 #print u_n
                 kappa = kappa +self.weights[m]*self.jac_s[k,m]*u_n
                 
        return kappa
    
        
    def surface_unit_normal(self):
        n_x_f, n_y_f = space_f90.surface_1_2d(self.x,self.y,self.D)
        #n_x = np.zeros((self.noe, self.n, self.n),float)
        #n_y = np.zeros((self.noe, self.n, self.n),float)
        n_x = np.zeros((self.noe_bc, self.n),float)
        n_y = np.zeros((self.noe_bc, self.n),float)
        
       
        for k in range(self.noe_bc):
            for m in range(self.n):
                [i, phys, boundarycondtion_type, element_edge] = self.theta_phys_bc(k,m)
                if element_edge[1] == 0:
                    k_e = element_edge[0]
                    #n_x[k_e,m,0]       =  n_x_f[k_e,0,m]
                    #n_y[k_e,m,0]       =  n_y_f[k_e,0,m]
                    n_x[k,m]       =  n_x_f[k_e,0,m]
                    n_y[k,m]       =  n_y_f[k_e,0,m]
                        
                if element_edge[1] == 1:
                    k_e = element_edge[0]
                    #n_x[k_e,m,self.n-1] =  n_x_f[k_e,1,m]
                    #n_y[k_e,m,self.n-1] =  n_y_f[k_e,1,m]
                    n_x[k,m] =  n_x_f[k_e,1,m]
                    n_y[k,m] =  n_y_f[k_e,1,m]
                        
                if element_edge[1] == 2:
                    k_e = element_edge[0]
                    #n_x[k_e,0,m]        =  n_x_f[k_e,2,m]
                    #n_y[k_e,0,m]        =  n_y_f[k_e,2,m]
                    n_x[k,m]        =  n_x_f[k_e,2,m]
                    n_y[k,m]        =  n_y_f[k_e,2,m]
                        
                if element_edge[1] == 3:
                    k_e = element_edge[0]
                    #n_x[k_e,self.n-1,m] = n_x_f[k_e,3,m]
                    #n_y[k_e,self.n-1,m] = n_y_f[k_e,3,m]
                    n_x[k,m] = n_x_f[k_e,3,m]
                    n_y[k,m] = n_y_f[k_e,3,m]
                
        #plt.figure()
        ##k=2
        #for k in range(self.noe):
        #    Q = plt.quiver(self.x[k,:,:],self.y[k,:,:],n_x[k,:,:],n_y[k,:,:],\
        #                   scale=25.0)#,units='dots')#,color='r',linewidths=(2,) )
        ##qk =plt.quiverkey(Q, 0.9, 0.95, 2,'asdf',coordinates='figure')
        ##plt.axis([-0.2,1.2,-0.2,1.2])
        #plt.show()
        return n_x,n_y
                
                
        
    def boundary_value(self,u):
        '''
        Creates an array of boundary values of a function u.
        
        :param u: An array in local data representation
        
        :returns: An array of boundary points.
        '''
        u_bound = np.zeros((self.noe_bc, self.n), float)
        
        for k in range(self.noe_bc):
            for m in range(self.n):
                [i, phys, bc_type, element_edge] = self.theta_phys_bc(k,m)
                k_element = element_edge[0]
                if element_edge[1] == int(0):
                    u_bound[k,m] = u[k_element,m,0]
                if element_edge[1] == int(1):
                    u_bound[k,m] = u[k_element,m,self.n-1]
                if element_edge[1] == int(2):
                    u_bound[k,m] = u[k_element,0,m]
                if element_edge[1] == int(3):
                    u_bound[k,m] = u[k_element,self.n-1,m]
                
        return u_bound
        
    def mapping_q(self,v):
        '''
        The mapping operator :math:`Q`.
        
        :param v: Numpy array in global data representation.
        :returns: Numpy array in local data representation
        
        The method maps a vector of global ordering to a local data 
        representation. This operator is used in the assembly or the 
        action of a global matrix on a vector,
        
        .. math::
           \underline w=\underline  Q^T\underline A_L\underline Q\underline v=\underline A\,\underline v
           
        where :math:`\underline A_L` is e.g. the local Laplacian or the local mass matrix. 
        
           
        Given that a mesh object has already been created one can perform the matrix vector 
        product :math:`\underline v_{local}=\underline Q\underline v_{global}` by typing::
        
           v_local = mesh.mapping_Q(v_global)
        
        '''
        #w = np.zeros((self.noe,self.n,self.n),float)
        #for k in range(self.noe):
        #    for m in range(self.n):
        #        for n in range(self.n):
        #            i = self.theta[k,m,n]
        #            w[k,m,n] = v[i]
        w = space_f90.mapping_q_2d(v,self.theta)
        return w
        
    def mapping_qt(self,w):
        '''
        The transpose :math:`Q^T` of the mapping operator :math:`Q`. This method maps an array 
        of local ordering to a global data representation. 
        
        :param w: Numpy array in local data representation.
        :returns: Numpy array in global data representation 
        
        The matrix vector product :math:`v_{global}=Q^Tv_{local}`::
        
           v_global = mesh.mapping_Q(v_local)
        
        '''
        #v = np.zeros(self.dof,float)
        #for k in range(self.noe):
        #    for m in range(self.n):
        #        for n in range(self.n):
        #            i = self.theta[k,m,n]
        #            v[i] = v[i] + w[k,m,n]
        v = space_f90.mapping_qt_2d(w,self.theta,self.dof)
        return v

    def local_to_global(self,w):
        '''
        Maps a function in H1 from local to global
        '''
        #v = np.zeros(self.dof,float)
        #for k in range(self.noe):
        #    for m in range(self.n):
        #        for n in range(self.n):
        #            i = self.theta[k,m,n]
        #            v[i] = w[k,m,n]
        v = space_f90.local_to_global_2d(w,self.theta,self.dof)
        return v
        
    def mask(self,w):
        '''
        This method masks nodal point values associated with a Dirichlet boundary condition.
        
        :param w: Numpy array in global data representation subject to masking.
        :returns: A masked numpy array in global data representation.
        
        '''
        for k in range(self.noe_bc):
            for m in range(self.n):
                [i, name, bc_type, edge] = self.theta_phys_bc(k,m)
                if bc_type == 'Dir':
                    w[i] = 0.0
        return w
        
    def mask_matrix(self,A):
        '''
        This method masks nodal point values associated with a Dirichlet boundary condition.
        
        :param A: Numpy array in global data representation subject to masking.
        :type A: PysparseMatrix
        :returns: A masked PysparseMatrix.
        
        '''
        
        for k in range(self.noe_bc):
            for m in range(self.n):
                [i, phys, bc_type, element_edge]=self.theta_phys_bc(k,m)
                if bc_type == "Dir":
                    A[int(i),:] = 0.0
        return A
        
    def interpolation(self,u,m):
        '''
        Interpolates a function :math:`u_N\\in\\mathbb P_N(\\Omega)`
        to a function :math:`u\\in\\mathbb P_{m-1}(\\Omega)`.
        
        .. math::
           u=I_N^{m-1}u_N
        
        :param u: Basis coefficients of a function with polynomial degree N.
        :param m: Polynomial degree of the interpolated function.
        :returns: Basis coefficients of the interpolated functon :math:`u_{m-1}`. 
        '''
        #print 'oobs look at X.interpolation(u,m)'
        #print 'in the Mesh2D class, remove u_int=np.zeros()?'
        #u_int = np.zeros((self.noe, self.n, self.n),float)
        
        n_local = u.shape[1]
        points_local, weights_local = \
                        self.basic.gauss_lobatto_legendre(n_local)
        points_int,   weights_int   = \
                        self.basic.gauss_lobatto_legendre( m )
        u_int = space_f90.interpolation_2d(points_int,u,points_local)
        return u_int
        
    def hyper_filter(self,u_tilde,alpha=0.25):
        '''
        A filter for hyperbolic problems (see Deville et al.). 
        
        .. math::
           u = F_\\alpha \\tilde u
           
        where the filter is
        
        .. math::
           F_\\alpha := \\alpha \\Pi_{N-1} + (1-\\alpha) I_N^N,\quad 
           \\Pi_{N-1} = I_{N-1}^N I_{N}^{N-1}
           
        Where :math:`\\alpha` is typically :math:`0.05<\\alpha<0.3`. 
           
        :param u_tilde: Basis coefficients of function to be filtered. 
        :param alpha: Weighting factor.
        :returns: Basis coefficients of the filtered function. 
           
        '''
        #for i in range()
        #La = Lagrange(i,n,x,gll_points)
        
        u1 = self.interpolation(u_tilde,self.n-1)
        u2 = self.interpolation(u1,self.n)
        u_out = alpha * u2 + (1.0 - alpha) * u_tilde
        return u_out
        
    def connectivity(self):
        theta, dof = space_f90.connectivity_2d_v2(self.x,self.y)
        return theta, dof
        

    def connectivity_python(self):
        
        tol = 1.0e-10
        theta = np.zeros((self.noe,self.n,self.n),int)
        # First element
        count = 0
        for m in range(self.n):
            for n in range(self.n):
                theta[0,m,n] = count
                count = count + 1
        # Remaining elements
        count=theta.max()
        for k in range(1,self.noe):
            for i in range(self.n):
                for j in range(self.n):
                    x_loc=self.x[k,i,j]
                    y_loc=self.y[k,i,j]
                    count=count+1
                    theta[k,i,j]=count
                    alpha=0
                    for kk in range(0,k):
                        for ii in range(self.n):
                            for jj in range(self.n):
                                #if x_loc == self.x[kk,ii,jj] and y_loc == self.y[kk,ii,jj]:
                                if abs(x_loc - self.x[kk,ii,jj])< tol and \
                                   abs(y_loc - self.y[kk,ii,jj]) < tol:
                                       theta[k,i,j] = theta[kk,ii,jj]
                                       count = count - 1
                                       alpha = 1
                                       break
                            if alpha == 1:
                                break
                        if alpha == 1:
                            break
        dof = theta.max() + 1
        print 'mesh dof=',dof
        return theta, dof

    def boundary_connectivity_gmsh(self,j):
        u      = np.zeros((self.GTS.dof),float)
        noe_bc = self.GTS.theta_bc.shape[0]
        
        for k in range(noe_bc):
            for m in range(self.GTS.n):
                i = self.GTS.theta_bc[k,m]
                if self.GTS.physical_bc[k] == j:
                    u[i] = self.GTS.physical_bc[k]
                    
        return self.GTS.mapping_q(u)
        
         
    def boundary_connectivity_gll(self,u_gmsh):
        #u_gll = np.zeros((self.noe,self.n,self.n),float)
        #for k in range(self.noe):
        #    for i in range(self.n):
        #        for j in range(self.n):
        #            for m in range(self.GTS.n):
        #                for n in range(self.GTS.n):
        #                    L_xi  = basic.lagrange(m,self.GTS.n,self.points[i],self.GTS.points)
        #                    L_eta = basic.lagrange(n,self.GTS.n,self.points[j],self.GTS.points)
        #                    u_gll[k,i,j] = u_gll[k,i,j]+u_gmsh[k,m,n]*L_xi*L_eta
        u_gll = self.interpolation(u_gmsh,self.n)
        return np.rint(u_gll)
        
    def boundary_connectivity_theta(self, theta_bc, physical_bc, jac_s, edge, count, u_gll, i):
        #
        for k in range(self.noe):
                alpha1=0.0
                alpha2=0.0
                alpha3=0.0
                alpha4=0.0
                for n in range(self.n):
                    alpha1=alpha1 + u_gll[k,n,0]
                    alpha2=alpha2 + u_gll[k,n,self.n-1]
                    alpha3=alpha3 + u_gll[k,0,n]
                    alpha4=alpha4 + u_gll[k,self.n-1,n]
                if int(np.rint(alpha1)) == self.n*i:
                    for n in range(self.n):
                        theta_bc[count,n]=self.theta[k,n,0]# edge 0
                        physical_bc[count]=i
                        # The corresponding surface Jacobian
                        x_xi=0.0;y_xi=0.0
                        for m in range(self.n):
                            x_xi  = x_xi  + self.D[n,m] * self.x[k,m,0]
                            y_xi  = y_xi  + self.D[n,m] * self.y[k,m,0]
                        jac_s[count,n] = np.sqrt(x_xi*x_xi + y_xi*y_xi)
                        # Edge
                        edge[count,0] = k # element number
                        edge[count,1] = 0 # edge number
                    count = count +1
                if int(np.rint(alpha2)) == self.n*i:
                    for n in range(self.n):
                       theta_bc[count,n]=self.theta[k,n,self.n-1] # edge 1
                       physical_bc[count]=i
                       # The corresponding surface Jacobian
                       x_xi=0.0;y_xi=0.0
                       for m in range(self.n):
                           x_xi  = x_xi  + self.D[n,m] * self.x[k,m,self.n-1]
                           y_xi  = y_xi  + self.D[n,m] * self.y[k,m,self.n-1]
                       jac_s[count,n] = np.sqrt(x_xi*x_xi + y_xi*y_xi)
                       # Edge
                       edge[count,0] = k # element number
                       edge[count,1] = 1 # edge number
                    count = count +1
                if int(np.rint(alpha3)) == self.n*i:
                    for n in range(self.n):
                        theta_bc[count,n]=self.theta[k,0,n] # edge 2
                        physical_bc[count]=i
                        # The corresponding surface Jacobian
                        x_xi=0.0;y_xi=0.0
                        for m in range(self.n):
                            x_xi  = x_xi  + self.D[n,m] * self.x[k,0,m]
                            y_xi  = y_xi  + self.D[n,m] * self.y[k,0,m]
                        jac_s[count,n] = np.sqrt(x_xi*x_xi + y_xi*y_xi)
                        # Edge
                        edge[count,0] = k # element number
                        edge[count,1] = 2 # edge number
                    count = count +1
                if int(np.rint(alpha4)) == self.n*i:
                    for n in range(self.n):
                        theta_bc[count,n]=self.theta[k,self.n-1,n] # edge 3
                        physical_bc[count]=i
                        # The corresponding surface Jacobian
                        x_xi=0.0;y_xi=0.0
                        for m in range(self.n):
                            x_xi  = x_xi  + self.D[n,m] * self.x[k,self.n-1,m]
                            y_xi  = y_xi  + self.D[n,m] * self.y[k,self.n-1,m]
                        jac_s[count,n] = np.sqrt(x_xi*x_xi + y_xi*y_xi)
                        # Edge
                        edge[count,0] = k # element number
                        edge[count,1] = 3 # edge number
                    count = count + 1
                    
        return theta_bc, physical_bc, jac_s, edge, count
        
    def boundary_connectivity(self):
        noe_bc      = self.GTS.theta_bc.shape[0]
        theta_bc    = np.zeros((noe_bc,self.n), int)
        jac_s       = np.zeros((noe_bc,self.n), float)
        u_gll       = np.zeros((self.noe,self.n,self.n), float)
        physical_bc = np.zeros((noe_bc), int)
        edge        = np.zeros((noe_bc, 2), int)
        
        # Locating boundary points
        count=0
        for i in range(self.GTS.physical_names.shape[0]):
            if self.GTS.physical_names[i,0] == 1:
                j = self.GTS.physical_names[i,1]
                u_gmsh = self.boundary_connectivity_gmsh(j)
                u_gll  = self.boundary_connectivity_gll(u_gmsh)
                theta_bc, physical_bc, jac_s, edge, count = \
                    self.boundary_connectivity_theta(theta_bc, physical_bc, jac_s,\
                                                     edge, count, u_gll, j)

        return theta_bc, physical_bc, jac_s, edge
        





import sempy
import numpy as np
import scipy.sparse.linalg as spl

import sempy.basic.basic_f90 as basic_f90

import matplotlib.pyplot as plt
import matplotlib.ticker as ticker
from mpl_toolkits.mplot3d import axes3d
from matplotlib import cm

from matplotlib import rcParams
rcParams['text.usetex']=True
rcParams['text.latex.unicode']=True

import time

from projection import ProjectionMethod
from projection import bdf1ex1
from projection import bdf2ex2
from projection import bdf3ex3
from projection import cnab2ex2
from projection import __bc__

def erk4():
    '''
    Explicit RK method of order 4.
    '''
    ah,bh,ch =np.zeros((4,4),float),np.zeros((4),float),np.zeros((4),float)

    ah[1,0]=1.0/2.0
    ah[2,1]=1.0/2.0
    ah[3,2]=1.0

    ch[1],ch[2],ch[3]=ah[1,0],ah[2,1],ah[3,2]
    bh[0],bh[1],bh[2],bh[3]=1.0/6.0 ,1.0/3.0, 1.0/3.0, 1.0/6.0
    stages=4
    return ah,bh,ch,stages 

class PC_OIFS(ProjectionMethod):
    '''
    OIFS.
    '''
    def __init__( self, Space, SpaceGL, Time, u, p, T= None,  
                  mu = None , kappa = None, Ri = 0.0,
                  order = 1, conv_term = False,
                  bound_cond = __bc__ , force_function = None,
                  init_steps = 1, explicit_steps =1, 
                  neumann_cond = None, chi = 0.5,
                  deflation_solver = True, call_back = None,
                  hyper_filter = False,
                  isothermal = True, 
                  tol = 1e-09, file_increment =100):
        
        self.order = order
        if self.order == 1:
            method = bdf1ex1
        if self.order == 2:
            method = bdf2ex2
            #method = cnab2ex2
        
        ProjectionMethod.__init__( self, Space, SpaceGL, Time, u, p, 
                                   T= T,  
                  mu = mu , kappa = kappa, Ri = Ri,
                  method = method, conv_term = conv_term,
                  bound_cond = bound_cond, force_function = force_function,
                  init_steps = init_steps, 
                  neumann_cond = neumann_cond, chi = chi,
                  deflation_solver = deflation_solver, 
                  call_back = call_back,
                  hyper_filter = hyper_filter,
                  isothermal = isothermal, 
                  tol = tol, file_increment = file_increment)
        
        self.explicit_steps = explicit_steps
        self.u_star = sempy.VectorFunction( self.Space )
        
        # Explicit solver
        u_temp = [ ]
        for i in range(self.Space.dim):
            u_temp.append( sempy.Function( self.Space ) )
        self.erk_u = sempy.ode.PRK( u_temp, self.Time, 
                                    force_function = self.force_function_u,
                                    bound_cond = self.bound_cond_u,
                                    prk_method = 'erk4',silent=True)
        if not self.isothermal:
            T_temp = sempy.Function( self.T.Space )
            self.erk_T = sempy.ode.PRK( [T_temp], self.Time, 
                                     force_function = self.force_function_T,
                                     bound_cond = self.bound_cond_T,
                                     prk_method = 'erk4',silent=True)
        
    def bound_cond_u(self,u,t):
        T_temp = sempy.Function(self.Space,basis_coeff=0.0)
        u,T_temp = self.bound_cond(u, T_temp, t)
        return u
    
    def bound_cond_T(self,T,t):
        u_temp = sempy.VectorFunction(self.Space)
        u_temp,T[0] = self.bound_cond(u_temp, T[0], t)
        return T
    
    def force_function_u( self, u, t ):
        # Forcing term f
        T_temp = sempy.Function(self.Space,basis_coeff=0.0)
        p_temp = sempy.Function(self.SpaceGL,basis_coeff=0.0)
        s = self.force_function( u, p_temp, T_temp,t )
        
        F = np.zeros((self.Space.dim, self.Space.dof),float)
        for i in range(self.Space.dim):
            F[i][:]= self.M*s[i][:]
        
        # Convection term
        self.inter_extra_polate(t)
        # Convection operator
        if self.Space.dim ==1:
            self.conv.u_conv = [ self.u_star[0].basis_coeff ]
        if self.Space.dim ==2:
            self.conv.u_conv = [ self.u_star[0].basis_coeff,
                                 self.u_star[1].basis_coeff ]
        if self.Space.dim ==3:
            self.conv.u_conv = [ self.u_star[0].basis_coeff,
                                 self.u_star[1].basis_coeff, 
                                 self.u_star[2].basis_coeff ]
        
        q = np.zeros((self.Space.dof),float)    
        for j in range( self.Space.dim):
            q = self.conv.matrix * u[j].glob()
            F[j][:] = F[j][:] - q[:]
        return F
    
    def force_function_T( self, T, t ):
        # Forcing term f
        #T_temp = sempy.Function(self.Space,basis_coeff=0.0)
        #p_temp = sempy.Function(self.SpaceGL,basis_coeff=0.0)
        #s = self.force_function( u, p_temp, T_temp,t )
        
        F = np.zeros((self.T.Space.dof),float)
        #for i in range(self.Space.dim):
        #    F[i][:]= self.M*s[i][:]
        
        # Convection term
        self.inter_extra_polate(t)
        # Convection operator
        if self.Space.dim ==1:
            self.conv_T.u_conv = [ self.u_star[0].basis_coeff ]
        if self.Space.dim ==2:
            self.conv_T.u_conv = [ self.u_star[0].basis_coeff,
                                   self.u_star[1].basis_coeff ]
        if self.Space.dim ==3:
            self.conv_T.u_conv = [ self.u_star[0].basis_coeff,
                                   self.u_star[1].basis_coeff, 
                                   self.u_star[2].basis_coeff ]
        
        q = np.zeros((self.T.Space.dof),float)    
        #for j in range( self.Space.dim):
        q[:] = self.conv_T.matrix * T[0].glob()
        F[:] = - q[:]
        return [F]
    
    def __solve_momentum( self, u, u_tilde, p, T, T_tilde ):
        '''
        '''
        # Solve the energy equation:
        if not self.isothermal:
            # 1) Convection subproblems
            for j in range(self.levels-1):
                T_temp = self.solve_nonlinear_T( self.Time.time,j )
                T_tilde[j].basis_coeff = np.copy(T_temp[0].basis_coeff)
            #if self.Time.time > 0.2:
            #    print 'o T=',T_tilde[0].basis_coeff[1,3,4]
            #    print '1 T=',T_tilde[1].basis_coeff[1,3,4]
            #for i in range(self.Space.dim):
            #    u_tilde[j][i].basis_coeff = np.copy(u_temp[i].basis_coeff)
            # 2) Diffusion subproblems
            F = self.__rhs_T__( T, u, T_tilde )
            s, flag = self.helmholtz_solver_T.solve( self.H_T, F, 
                                            T[self.levels-1].glob() )
            T[self.levels-1].basis_coeff = self.T.Space.mapping_q( s )
            print 'T flag=',flag
            
        # Solve the momentum equations:
        # 1) Convection subproblems
        t1_ex=time.time()
        for j in range(self.levels-1):
            u_temp = self.solve_nonlinear( self.Time.time,j )
            for i in range(self.Space.dim):
                u_tilde[j][i].basis_coeff = np.copy(u_temp[i].basis_coeff)
        t2_ex=time.time()
        print 'Convection ex time=',t2_ex-t1_ex
        # 2) Diffusion subproblems
        F = self.__rhs__( u, u_tilde,p, T )
        for i in range(self.Space.dim):
            s, flag = self.helmholtz_solver.solve( 
                                            self.H, F[i], 
                                            u[self.levels-1][i].glob() )
            u[self.levels-1][i].basis_coeff = self.Space.mapping_q( s )
            print 'i=',i,'flag=',flag
        return u, T
    
    def inter_extra_polate(self,t):
        '''
        Interpolation/extrapolation of the velocity field.
        '''
        if self.order == 1:
            for i in range(self.Space.dim):
                self.u_star[i].basis_coeff = np.copy(
                            self.u_levels[self.levels-2][i].basis_coeff)
        if self.order == 2:
            for i in range(self.Space.dim):
                u_int = self.u_levels[self.levels-2][i].basis_coeff-\
                    (self.u_levels[self.levels-2][i].basis_coeff -\
                     self.u_levels[self.levels-3][i].basis_coeff)/(self.Time.h)*\
                    (self.Time.time-t-self.Time.h)
                    #(t-self.Time.time-self.Time.h)
                self.u_star[i].basis_coeff = np.copy(u_int)
                                       
    
    def solve_nonlinear(self,t_global,j):
        '''
        Solve the convection sub-problems.
        '''
        h=self.Time.h
        # Time domain
        Time_local = sempy.ode.Time(self.Time.Space,
                        start_time = t_global - np.float(self.levels-j-1)*h,
                        end_time = t_global,
                        time_steps = self.explicit_steps*(self.levels-j-1))
        self.erk_u.Time = Time_local
        # Initial condition
        for k in range(self.Space.dim):
            self.erk_u.y0[k].basis_coeff = np.copy(
                                        self.u_levels[j][k].basis_coeff)
        # Solve the problem
        y_temp = self.erk_u.solve()
        
        return y_temp
       
    def solve_nonlinear_T(self,t_global,j):
        '''
        Solve the convection sub-problems.
        '''
        h=self.Time.h
        # Time domain
        Time_local = sempy.ode.Time(self.Time.Space,
                        start_time = t_global - np.float(self.levels-j-1)*h,
                        end_time = t_global,
                        time_steps = self.explicit_steps*(self.levels-j-1))
        self.erk_T.Time = Time_local
        # Initial condition
        #for k in range(self.Space.dim):
        self.erk_T.y0[0].basis_coeff = np.copy(self.T_levels[j].basis_coeff)
        # Solve the problem
        y_temp = self.erk_T.solve()
        
        return y_temp        
    
    def __rhs__(self, u, u_tilde,p, T ):
        '''
        The right hand side of the momentum equations.
         
        :param u: Velocity
        :param p: Pressure
        '''
        # RHS
        if self.Space.dim == 1:
            F1 = np.zeros( ( self.Space.dof ), float )
            F = [ F1 ]
        if self.Space.dim == 2:
            F1 = np.zeros( ( self.Space.dof ), float )
            F2 = np.zeros( ( self.Space.dof ), float )
            F = [ F1, F2 ]
        
        if self.Space.dim == 3:
            F1 = np.zeros( ( self.Space.dof ), float )
            F2 = np.zeros( ( self.Space.dof ), float )
            F3 = np.zeros( ( self.Space.dof ), float )
            F = [ F1, F2, F3 ]
                
        s = np.zeros( ( self.Space.dof ), float )
        
        # Contribution from temporal derivative
        for j in range( self.Space.dim ) : 
            for i in range( self.levels - 1 ) :   
                if abs( self.alpha[i]) > 0 :
                    #F[j] = F[j] - self.alpha[i] * self.M * u[i][j].glob()
                    F[j] = F[j] - self.alpha[i]*self.M*u_tilde[i][j].glob()
        
        # Contribution from the buoyancy term
        if np.abs(self.Ri) > 1e-15:
            for i in range( self.levels ) :
                if abs( self.beta[i]) > 0 :
                    coeff = self.Time.h * self.beta[i] * self.Ri
                    if self.Space.dim == 1:# and j == 0:
                        F[0] = F[0] + coeff * self.M * T[i].glob()
                    if self.Space.dim == 2:# and j == 1:
                        F[1] = F[1] + coeff * self.M * T[i].glob()            
                    if self.Space.dim == 3:# and j == 2:
                        F[2] = F[2] + coeff * self.M * T[i].glob()
                    
        # Contribution from the viscous term:
        for j in range( self.Space.dim ) :
            for i in range( self.levels - 1 ) : 
                if abs( self.beta[i] ) > 0 :
                    s = self.A * u[i][j].glob()
                    F[j] = F[j] - self.Time.h * self.beta[i] * s
        
        
        # Contribution from the linear form stemming from the Laplacian
        for i in range( self.levels ) :
            if abs( self.beta[i] ) > 0 :
                local_time = self.Time.time - \
                             ( float( self.levels ) - 1.0 - \
                             float(i) ) * self.Time.h
                s1 = self.neumann_cond( local_time )
                for j in range( self.Space.dim ):
                    self.lap.g_neu = s1[j]
                    s = self.lap.assemble_linear_form()
                    F[j] = F[j] + self.Time.h * self.beta[i] * s
                    
        # Pressure gradient
        for j in range( self.Space.dim ) :
            for i in range( self.levels ) :
                if abs( self.eta[i] ) > 0 :
                    s = self.G[j] * p[i].glob()
                    F[j] = F[j] - self.Time.h * self.eta[i] * s
        
        
        # Forcing term
        #for i in range( self.levels ):
        #    if abs(self.gamma[i]) > 0 :
        #        local_time = self.Time.time - \
        #                     ( float( self.levels ) - 1.0 - \
        #                     float(i) ) * self.Time.h
        #        s = self.force_function( u[i], p[i], T[i],local_time )
        #        for j in range( self.Space.dim ) :
        #            F[j] = F[j] + \
        #                   self.Time.h * self.gamma[i] * self.M * s[j]
         
        return F
    
    def __rhs_T__(self, T, u, T_tilde ):
        '''
        The right hand side of the momentum equations.
         
        :param T: Temperature.
        :param u: Velocity.
        '''
        # RHS
        F = np.zeros( ( self.Space.dof ), float )
        s = np.zeros( ( self.Space.dof ), float )
        
        # Contribution from temporal derivative
        for i in range( self.levels - 1 ) :
            if abs( self.alpha[i]) > 0 :
                #F[:] = F[:] - self.alpha[i] * self.M_T * T[i].glob()
                F[:] = F[:] - self.alpha[i] * self.M_T * T_tilde[i].glob()
                    
        # Contribution from the viscous term:
        for i in range( self.levels - 1 ) :
            if abs( self.beta[i] ) > 0 :
                s[:] = self.A_T * T[i].glob()
                F[:] = F[:] - self.Time.h * self.beta[i] * s[:]
        
        # Contribution from the linear form stemming from the Laplacian
        #for i in range( self.levels ) :
        #    if abs( self.beta[i] ) > 0 :
        #        local_time = self.Time.time - \
        #                     ( float( self.levels ) - 1.0 - \
        #                     float(i) ) * self.Time.h
        #        s1 = self.neumann_cond( local_time )
        #        for j in range( self.Space.dim ):
        #            self.lap.g_neu = s1[j]
        #            s = self.lap.assemble_linear_form()
        #            #print 'j=',j,'s=',s 
        #            F[j] = F[j] + self.Time.h * self.beta[i] * s
                    
        # Convection term
        #if self.conv_term :
        #    for i in range( self.levels - 1 ) :
        #        if self.Space.dim ==1:
        #            self.conv_T.u_conv = [ u[i][0].basis_coeff ]
        #        if self.Space.dim ==2:
        #            self.conv_T.u_conv = [ u[i][0].basis_coeff, 
        #                                   u[i][1].basis_coeff ]
        #        if self.Space.dim ==3:
        #            self.conv_T.u_conv = [ u[i][0].basis_coeff,
        #                                   u[i][1].basis_coeff, 
        #                                   u[i][2].basis_coeff ]
        #        if abs( self.gamma[i] ) > 0.0 :
        #            s[:] = self.conv_T.matrix * T[i].glob()
        #            F[:] = F[:] - self.Time.h * self.gamma[i] * s
         
        return F
        
    def solve(self):
        '''
        Solve the problem. 
        
        :returns: * **u** - Velocity
                  * **p** - Pressure
        '''
        delta = self.space_check()
        if delta > 1.0e-04:
            print 'Spaces for u and T are not compatible.'
            return self.u, self.p, self.T
        
        # Solution functions:
        self.u_levels = [ ]
        self.p_levels = [ ]
        self.T_levels = [ ]
        
        for i in range(self.levels):
            self.u_levels.append( sempy.VectorFunction( self.Space ) )
            self.p_levels.append( sempy.Function( self.SpaceGL ) )
            self.T_levels.append( sempy.Function( self.T.Space ) )
        
        # u tilde
        u_tilde = [ ]
        for i in range(self.levels-1):
            if self.Space.dim==1:
                u_tilde.append( sempy.Function( self.Space ) )
            if self.Space.dim==2:
                u_tilde.append( [sempy.Function( self.Space ),
                                sempy.Function( self.Space )] )
            if self.Space.dim==3:
                u_tilde.append( [sempy.Function( self.Space ),
                                 sempy.Function( self.Space ),
                                 sempy.Function( self.Space )] )
        T_tilde = [ ]
        for i in range(self.levels-1):
            T_tilde.append( sempy.Function( self.T.Space ) )
        # Setting initial values:
        for i in range(self.levels):
            self.p_levels[i].basis_coeff = np.copy( self.p.basis_coeff )
            for j in range(self.Space.dim) :
                self.u_levels[i][j].basis_coeff = np.copy( self.u[j].basis_coeff )
            self.T_levels[i].basis_coeff = np.copy( self.T.basis_coeff )
        # Timesteps to file:
        k_file = range( self.file_increment, 
                        self.Time.time_steps+self.file_increment, 
                        self.file_increment )
        
        # Initial value to file:        
        if not self.u.filename == 'none':
            self.u.to_file()
        if not self.p.filename == 'none':
            self.p.to_file()
        if not self.T.filename == 'none':
            self.T.to_file()
        # Callback
        if not self.call_back == None:
            self.call_back(self.u,self.p,self.T,self.Time.time,0)
        #-- Initialize
        factor = 0.0
        for i in range( 0, self.levels-2 ):
            print '--> Time step number x: ', i+1,'time=',self.Time.time
            self.Time.increment()
            factor = factor + 1.0
            u_dum, p_dum, T_dum = self.__initialize( 
                                        self.u_levels[i], 
                                        self.p_levels[i], 
                                        self.T_levels[i],
                                        ( factor - 1.0 ) * self.Time.h,
                                        factor*self.Time.h )
            self.p_levels[i+1].basis_coeff = np.copy( p_dum.basis_coeff )
            self.T_levels[i+1].basis_coeff = np.copy( T_dum.basis_coeff )
            for j in range(self.Space.dim) :
                self.u_levels[i+1][j].basis_coeff = np.copy( 
                                                u_dum[j].basis_coeff )
            del u_dum, p_dum, T_dum
            # -- Callback
            if not self.call_back == None:
                for j in range(self.Space.dim):
                    self.u[j].basis_coeff = np.copy( 
                                        self.u_levels[i+1][j].basis_coeff )
                self.p.basis_coeff = np.copy(self.p_levels[i+1].basis_coeff)
                self.T.basis_coeff = np.copy(self.T_levels[i+1].basis_coeff)
                self.call_back(self.u, self.p, self.T,self.Time.time, i+1)
            # To file
            if (i+1) in k_file:
                self.to_file(self.u_levels[i+1],self.p_levels[i+1],
                             self.T_levels[i+1])
        # Time stepping
        for k in range( self.levels-2, self.Time.time_steps ):
            self.Time.increment()
            print '--> Time step number: ', k+1,'time=',self.Time.time
            
            # Momentum equations:
            # BC
            self.u_levels[self.levels-1], self.T_levels[self.levels-1]=\
                self.bound_cond( self.u_levels[self.levels-1], 
                                 self.T_levels[self.levels-1], 
                                 self.Time.time )
            # Solve
            self.u_levels, self.T_levels = self.__solve_momentum( 
                            self.u_levels, u_tilde,
                            self.p_levels, self.T_levels, T_tilde )
            
            # Solve the pressure Poisson equation
            phi = self.solve_pressure( self.u_levels )
            
            # Update pressure and velocity
            self.p_levels, self.u_levels = self.update( 
                            self.p_levels, phi, self.u_levels )
            
            # Update solution array
            for i in range( self.levels-1 ):
                self.p_levels[i].basis_coeff = np.copy( 
                                    self.p_levels[i+1].basis_coeff )
                for j in range(self.Space.dim) :
                    self.u_levels[i][j].basis_coeff = np.copy( 
                                self.u_levels[i+1][j].basis_coeff )
                self.T_levels[i].basis_coeff = np.copy( 
                                self.T_levels[i+1].basis_coeff )
            # -- Callback
            if not self.call_back == None:
                for i in range(self.Space.dim):
                    self.u[i].basis_coeff = \
                             np.copy( self.u_levels[self.levels-1][i].basis_coeff )
                self.p.basis_coeff = \
                      np.copy(self.p_levels[self.levels-1].basis_coeff)
                self.T.basis_coeff = \
                      np.copy(self.T_levels[self.levels-1].basis_coeff)
                self.call_back(self.u,self.p,self.T,self.Time.time,k+1)
            # To file
            if (k+1) in k_file:
                print '-------->>>> to file'
                self.to_file(self.u_levels[self.levels-1],
                             self.p_levels[self.levels-1],
                             self.T_levels[self.levels-1])
                
        # Return values
        for i in range(self.Space.dim):
            self.u[i].basis_coeff = np.copy( 
                        self.u_levels[self.levels-1][i].basis_coeff )
        self.p.basis_coeff = np.copy(
                        self.p_levels[self.levels-1].basis_coeff)
        self.T.basis_coeff = np.copy(
                        self.T_levels[self.levels-1].basis_coeff)
        return self.u, self.p, self.T
    
    def __initialize( self, u_0, p_0, T_0, start, end ):
        #print 'initialize...'
        # Temporal domain
        t2 = sempy.ode.Time( self.Space,start_time = start,
                             end_time = end,
                             time_steps = self.init_steps )
        # Solution vectors 
        u_init = sempy.VectorFunction( self.Space )
        p_init = sempy.Function( self.SpaceGL )
        T_init = sempy.Function( self.T.Space )
    
        p_init.basis_coeff = np.copy( p_0.basis_coeff )
        T_init.basis_coeff = np.copy( T_0.basis_coeff )
        for j in range(self.Space.dim) :
            u_init[j].basis_coeff = np.copy( u_0[j].basis_coeff )
        
        eul = PC_OIFS( self.Space, self.SpaceGL, t2, 
                       u_init, p_init, T=T_init,  
                       mu = self.mu , kappa = self.kappa, 
                       Ri = self.Ri, order = 1, conv_term = self.conv_term,
                       bound_cond = self.bound_cond,  
                       force_function = self.force_function,
                       explicit_steps =self.explicit_steps, 
                       neumann_cond = self.neumann_cond, chi = 1.0,
                       deflation_solver = self.deflation_solver, 
                       hyper_filter = self.hyper_filter,
                       isothermal = self.isothermal,
                       tol = self.tol, file_increment =10000)
         
        u_init, p_init, T_init = eul.solve()
        del eul
        return u_init, p_init, T_init
       
        
class ProjectionOIFS(ProjectionMethod):
    '''
    OIFS.
    '''
    def __init__( self, Space, SpaceGL, Time, u, p, T= None,  
                  mu = None , kappa = None, Ri = 0.0,
                  order = 1, conv_term = False,
                  bound_cond = __bc__ , force_function = None,
                  init_steps = 1, explicit_steps =1, 
                  neumann_cond = None, chi = 0.5,
                  deflation_solver = True, call_back = None,
                  hyper_filter = False,
                  isothermal = True, 
                  tol = 1e-09, file_increment =100):
        
        self.order = order
        if self.order == 1:
            method = bdf1ex1
        if self.order == 2:
            method = bdf2ex2
        
        ProjectionMethod.__init__( self, Space, SpaceGL, Time, u, p, 
                                   T= T,  
                  mu = mu , kappa = kappa, Ri = Ri,
                  method = method, conv_term = conv_term,
                  bound_cond = bound_cond, force_function = force_function,
                  init_steps = 1, neumann_cond = neumann_cond, chi = chi,
                  deflation_solver = deflation_solver, 
                  call_back = call_back,
                  hyper_filter = hyper_filter,
                  isothermal = isothermal, 
                  tol = tol, file_increment = file_increment)
        
        self.explicit_steps = explicit_steps
        self.u_star = sempy.VectorFunction( self.Space )
    
    def __solve_momentum( self, u, p, T ):
        '''
        '''
        # Solve the energy equation:
        if not self.isothermal:
            F = self.__rhs_T__( T, u )
            s, flag = self.helmholtz_solver_T.solve( 
                                            self.H_T, F, 
                                            T[self.levels-1].glob() )
            T[self.levels-1].basis_coeff = self.T.Space.mapping_q( s )
            print 'T flag=',flag
            
        # Solve the momentum equations:
        # 1) Convection subproblems
        #self.solve_nonlinear(u[0],self.Time)
        #self.conv_sub(u)
        #self.__conv_sub()
        #self.__erk(u,p,T)
        #self.inter_extra_polate(u,self.Time.time)
        #self.u_star[0].plot_wire()
        #self.erk_timestepping(u,p,T)
        # 2) Diffusion subproblems
        F = self.__rhs__( u, p, T )
        for i in range(self.Space.dim):
            s, flag = self.helmholtz_solver.solve( 
                                            self.H, F[i], 
                                            u[self.levels-1][i].glob() )
            u[self.levels-1][i].basis_coeff = self.Space.mapping_q( s )
            print 'i=',i,'flag=',flag
        return u, T
    
    def inter_extra_polate(self,u,t):
        '''
        Interpolation/extrapolation of the velocity field.
        '''
        if self.order == 1:
            for i in range(self.Space.dim):
                self.u_star[i].basis_coeff = np.copy(
                                        u[self.levels-2][i].basis_coeff)
        if self.order == 2:
            for i in range(self.Space.dim):
                u_int = u[self.levels-2][i].basis_coeff+\
                       (u[self.levels-2][i].basis_coeff -\
                        u[self.levels-3][i].basis_coeff)/(self.Time.h)*\
                       (t-self.Time.time-self.Time.h)
                self.u_star[i].basis_coeff = np.copy(u_int)
                                       
    
    def non_linear_function(self,u_tilde,T_tilde,u,p,T,t):
        '''
        '''
        F  = np.zeros((self.Space.dim + 1,self.Space.dof),float)
        # Forcing term
        s = self.force_function( u, p, T, t )
        for j in range( self.Space.dim + 1):
            F[j][:]=self.M*s[j][:]
        # Interpolate/extrapolate velocity
        self.inter_extra_polate(u,t)
        # Convection operator
        if self.Space.dim ==1:
            self.conv.u_conv = [ self.u_star[0].basis_coeff ]
        if self.Space.dim ==2:
            self.conv.u_conv = [ self.u_star[0].basis_coeff,
                                 self.u_star[1].basis_coeff ]
        if self.Space.dim ==3:
            self.conv.u_conv = [ self.u_star[0].basis_coeff,
                                 self.u_star[1].basis_coeff, 
                                 self.u_star[2].basis_coeff ]
        #
        for j in range( self.Space.dim):
            s = self.conv.matrix * u_tilde[j].glob()
            F[j][:] = F[j][:] + s
            
        if not self.isothermal:
            if self.Space.dim ==1:
                self.conv_T.u_conv = [ self.u_star[0].basis_coeff ]
            if self.Space.dim ==2:
                self.conv_T.u_conv = [ self.u_star[0].basis_coeff, 
                                       self.u_star[1].basis_coeff ]
            if self.Space.dim ==3:
                self.conv_T.u_conv = [ self.u_star[0].basis_coeff,
                                       self.u_star[1].basis_coeff, 
                                       self.u_star[2].basis_coeff ]
            s = self.conv_T.matrix * T_tilde[j].glob()
            F[self.Space.dim][:] = F[self.Space.dim][:] + s[self.Space.dim]
            
        return F
    
    #def erk_solve(self):
        #-- Solution vector y: y[0]=y^{n+1}, y[1]=y^{n}, y[2]=y^{n-1}, 
        #u_tilde = np.zeros( (self.vector_size), float )
        #yend   = np.zeros( (self.vector_size), float )
        #ystart[:] = self.component_to_vector( self.y0 )
    
    def solve_nonlinear(self,u,Y):
        print 'hei'
        print 'self.levels=',self.levels
        # Solver
        Time_local = sempy.ode.Time(Y.Space,
                        start_time = 0.0,
                        end_time = Y.h,
                        time_steps = self.explicit_steps)
        if self.Space.dim==1:
            w=[u[0]]
        if self.Space.dim==2:
            w=[u[0],u[1]]
        if self.Space.dim==3:
            w=[u[0],u[1],u[2]]
        erk = sempy.ode.PRK( w,Time_local,
                             force_function = self.force_function, 
                             bound_cond = self.bound_cond,
                             prk_method='erk4')
        # Solve
        for j in range(self.levels-1):
            Time_local = sempy.ode.Time(Y.Space,
                start_time = Y.time - np.float(self.levels-j-1)*Y.h,
                end_time = Y.time,
                time_steps = self.explicit_steps*(self.levels-j-1))
        #    for k in range(self.Space.dim):
        #        erk.y0[k].basis_coeff = np.copy(y[j][k].basis_coeff)
                
        #    for k in range(self.number_of_eq):
        #        erk.y0[k].basis_coeff = np.copy(y[j][k].basis_coeff)
        #    #erk.y0 = y[j]
        #    erk.Time = Time_local
        #    y_temp = erk.solve()
        #    for k in range(self.number_of_eq):
        #        y_tilde[j][k].basis_coeff = np.copy(y_temp[k].basis_coeff)
        #    del y_temp 
            
    
    def __rhs__(self, u, p, T ):
        '''
        The right hand side of the momentum equations.
         
        :param u: Velocity
        :param p: Pressure
        '''
        # RHS
        if self.Space.dim == 1:
            F1 = np.zeros( ( self.Space.dof ), float )
            F = [ F1 ]
        if self.Space.dim == 2:
            F1 = np.zeros( ( self.Space.dof ), float )
            F2 = np.zeros( ( self.Space.dof ), float )
            F = [ F1, F2 ]
        
        if self.Space.dim == 3:
            F1 = np.zeros( ( self.Space.dof ), float )
            F2 = np.zeros( ( self.Space.dof ), float )
            F3 = np.zeros( ( self.Space.dof ), float )
            F = [ F1, F2, F3 ]
                
        s = np.zeros( ( self.Space.dof ), float )
        
        # Contribution from temporal derivative
        for j in range( self.Space.dim ) : 
            for i in range( self.levels - 1 ) :   
                if abs( self.alpha[i]) > 0 :
                    F[j] = F[j] - self.alpha[i] * self.M * u[i][j].glob()
        
        # Contribution from the buoyancy term
        if np.abs(self.Ri) > 1e-15:
            for i in range( self.levels ) :
                if abs( self.beta[i]) > 0 :
                    coeff = self.Time.h * self.beta[i] * self.Ri
                    if self.Space.dim == 1:# and j == 0:
                        F[0] = F[0] + coeff * self.M * T[i].glob()
                    if self.Space.dim == 2:# and j == 1:
                        F[1] = F[1] + coeff * self.M * T[i].glob()            
                    if self.Space.dim == 3:# and j == 2:
                        F[2] = F[2] + coeff * self.M * T[i].glob()
                    
        # Contribution from the viscous term:
        for j in range( self.Space.dim ) :
            for i in range( self.levels - 1 ) : 
                if abs( self.beta[i] ) > 0 :
                    s = self.A * u[i][j].glob()
                    F[j] = F[j] - self.Time.h * self.beta[i] * s
        
        
        # Contribution from the linear form stemming from the Laplacian
        for i in range( self.levels ) :
            if abs( self.beta[i] ) > 0 :
                local_time = self.Time.time - \
                             ( float( self.levels ) - 1.0 - \
                             float(i) ) * self.Time.h
                s1 = self.neumann_cond( local_time )
                for j in range( self.Space.dim ):
                    self.lap.g_neu = s1[j]
                    s = self.lap.assemble_linear_form()
                    #print 'j=',j,'s=',s 
                    F[j] = F[j] + self.Time.h * self.beta[i] * s
                    
        # Pressure gradient
        for j in range( self.Space.dim ) :
            for i in range( self.levels ) :
                if abs( self.eta[i] ) > 0 :
                    s = self.G[j] * p[i].glob()
                    F[j] = F[j] - self.Time.h * self.eta[i] * s
        
        # Forcing term
        for i in range( self.levels ):
            if abs(self.gamma[i]) > 0 :
                local_time = self.Time.time - \
                             ( float( self.levels ) - 1.0 - \
                             float(i) ) * self.Time.h
                s = self.force_function( u[i], p[i], T[i],local_time )
                for j in range( self.Space.dim ) :
                    F[j] = F[j] + \
                           self.Time.h * self.gamma[i] * self.M * s[j]
         
        return F
        
    def solve(self):
        '''
        Solve the problem. 
        
        :returns: * **u** - Velocity
                  * **p** - Pressure
        '''
        delta = self.space_check()
        if delta > 1.0e-04:
            print 'Spaces for u and T are not compatible.'
            return self.u, self.p, self.T
        print 'hei'
        
        # Solution functions:
        u = [ ]
        p = [ ]
        T = [ ]
        
        for i in range(self.levels):
            u.append( sempy.VectorFunction( self.Space ) )
            p.append( sempy.Function( self.SpaceGL ) )
            T.append( sempy.Function( self.T.Space ) )
        # Setting initial values:
        for i in range(self.levels):
            p[i].basis_coeff = np.copy( self.p.basis_coeff )
            for j in range(self.Space.dim) :
                u[i][j].basis_coeff = np.copy( self.u[j].basis_coeff )
            T[i].basis_coeff = np.copy( self.T.basis_coeff )
        # Timesteps to file:
        k_file = range( self.file_increment, 
                        self.Time.time_steps+self.file_increment, 
                        self.file_increment )
        
        # Initial value to file:        
        if not self.u.filename == 'none':
            self.u.to_file()
        if not self.p.filename == 'none':
            self.p.to_file()
        if not self.T.filename == 'none':
            self.T.to_file()
        # -- Callback
        if not self.call_back == None:
            self.call_back(self.u,self.p,self.T,self.Time.time,0)
        
        # Time stepping
        for k in range( self.levels-2, self.Time.time_steps ):
            print '--> Time step number: ', k+1,'time=',self.Time.time
            self.Time.increment()
            
            # Solve the momentum equations:
            self.bound_cond( u[self.levels-1], T[self.levels-1], 
                             self.Time.time )
            u, T = self.__solve_momentum( u, p, T )
            
                               
            # Solve the pressure Poisson equation
            phi = self.solve_pressure( u )
            
            # Update pressure and velocity
            p, u = self.update( p, phi, u )
            
            # Hyperbolic filter
            #if self.hyper_filter:
            #    for i in range(self.Space.dim):
            #        u[self.levels-1][i].hyper_filter()
            #    if not self.isothermal:
            #        T[self.levels-1].hyper_filter()
            #    self.bound_cond( u[self.levels-1], T[self.levels-1],
            #                     self.Time.time )
            
                    
            # Update solution array
            for i in range( self.levels-1 ):
                p[i].basis_coeff = np.copy( p[i+1].basis_coeff )
                for j in range(self.Space.dim) :
                    u[i][j].basis_coeff = np.copy( u[i+1][j].basis_coeff )
                T[i].basis_coeff = np.copy( T[i+1].basis_coeff )
            # -- Callback
            #if not self.call_back == None:
            #    for i in range(self.Space.dim):
            #        self.u[i].basis_coeff = \
            #                 np.copy( u[self.levels-1][i].basis_coeff )
            #    self.p.basis_coeff = \
            #          np.copy(p[self.levels-1].basis_coeff)
            #    self.T.basis_coeff = \
            #          np.copy(T[self.levels-1].basis_coeff)
            #    self.call_back(self.u,self.p,self.T,self.Time.time,k+1)
            # Print to file
            #if (k+1) in k_file:
            #    self.to_file(u[self.levels-1], p[self.levels-1], 
            #                 T[self.levels-1])
                
        #print 'end time=',self.Time.time
        # Return values
        for i in range(self.Space.dim):
            self.u[i].basis_coeff = np.copy( u[self.levels-1][i].basis_coeff )
        self.p.basis_coeff = np.copy(p[self.levels-1].basis_coeff)
        self.T.basis_coeff = np.copy(T[self.levels-1].basis_coeff)
        return self.u, self.p, self.T
        